@@ -1,20 +1,20 @@
 # encoding: utf-8
 
 '''CKAN environment configuration'''
-import json
 import os
 import logging
 import warnings
-from urlparse import urlparse
 import pytz
 
+import six
 import sqlalchemy
-from pylons import config as pylons_config
-import formencode
+
+from six.moves.urllib.parse import urlparse
 
 import ckan.config.routing as routing
 import ckan.model as model
 import ckan.plugins as p
+import ckan.lib.plugins as lib_plugins
 import ckan.lib.helpers as helpers
 import ckan.lib.app_globals as app_globals
 from ckan.lib.redis import is_redis_available
@@ -23,11 +23,16 @@ import ckan.lib.search as search
 import ckan.logic as logic
 import ckan.authz as authz
 import ckan.lib.jinja_extensions as jinja_extensions
+from ckan.lib.webassets_tools import webassets_init
 from ckan.lib.i18n import build_js_translations
 
 from ckan.common import _, ungettext, config
 from ckan.exceptions import CkanConfigurationException
 
+if six.PY2:
+    from pylons import config as pylons_config
+
+
 log = logging.getLogger(__name__)
 
 
@@ -35,54 +40,55 @@ log = logging.getLogger(__name__)
 warnings.simplefilter('ignore', UserWarning)
 
 
-def load_environment(global_conf, app_conf):
+def load_environment(conf):
     """
     Configure the Pylons environment via the ``pylons.config`` object. This
     code should only need to be run once.
     """
-    # this must be run at a time when the env is semi-setup, thus inlined here.
-    # Required by the deliverance plugin and iATI
-    from pylons.wsgiapp import PylonsApp
-    import pkg_resources
-    find_controller_generic = getattr(
-        PylonsApp.find_controller,
-        '_old_find_controller',
-        PylonsApp.find_controller)
+    if six.PY2:
+        # this must be run at a time when the env is semi-setup, thus inlined
+        # here. Required by the deliverance plugin and iATI
+        from pylons.wsgiapp import PylonsApp
+        import pkg_resources
+        find_controller_generic = getattr(
+            PylonsApp.find_controller,
+            '_old_find_controller',
+            PylonsApp.find_controller)
 
-    # This is from pylons 1.0 source, will monkey-patch into 0.9.7
-    def find_controller(self, controller):
-        if controller in self.controller_classes:
-            return self.controller_classes[controller]
-        # Check to see if its a dotted name
-        if '.' in controller or ':' in controller:
-            ep = pkg_resources.EntryPoint.parse('x={0}'.format(controller))
+        # This is from pylons 1.0 source, will monkey-patch into 0.9.7
+        def find_controller(self, controller):
+            if controller in self.controller_classes:
+                return self.controller_classes[controller]
+            # Check to see if its a dotted name
+            if '.' in controller or ':' in controller:
+                ep = pkg_resources.EntryPoint.parse('x={0}'.format(controller))
 
-            if hasattr(ep, 'resolve'):
-                # setuptools >= 10.2
-                mycontroller = ep.resolve()
-            else:
-                # setuptools >= 11.3
-                mycontroller = ep.load(False)
+                if hasattr(ep, 'resolve'):
+                    # setuptools >= 10.2
+                    mycontroller = ep.resolve()
+                else:
+                    # setuptools >= 11.3
+                    mycontroller = ep.load(False)
 
-            self.controller_classes[controller] = mycontroller
-            return mycontroller
-        return find_controller_generic(self, controller)
-    find_controller._old_find_controller = find_controller_generic
-    PylonsApp.find_controller = find_controller
+                self.controller_classes[controller] = mycontroller
+                return mycontroller
+            return find_controller_generic(self, controller)
+        find_controller._old_find_controller = find_controller_generic
+        PylonsApp.find_controller = find_controller
 
-    os.environ['CKAN_CONFIG'] = global_conf['__file__']
+    os.environ['CKAN_CONFIG'] = conf['__file__']
 
     # Pylons paths
     root = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
 
-    valid_base_public_folder_names = ['public', 'public-bs2']
-    static_files = app_conf.get('ckan.base_public_folder', 'public')
-    app_conf['ckan.base_public_folder'] = static_files
+    valid_base_public_folder_names = ['public']
+    static_files = conf.get('ckan.base_public_folder', 'public')
+    conf['ckan.base_public_folder'] = static_files
 
     if static_files not in valid_base_public_folder_names:
         raise CkanConfigurationException(
             'You provided an invalid value for ckan.base_public_folder. '
-            'Possible values are: "public" and "public-bs2".'
+            'Possible values are: "public".'
         )
 
     log.info('Loading static files from %s' % static_files)
@@ -92,16 +98,17 @@ def load_environment(global_conf, app_conf):
                  templates=[])
 
     # Initialize main CKAN config object
-    config.update(global_conf)
-    config.update(app_conf)
+    config.update(conf)
 
-    # Initialize Pylons own config object
-    pylons_config.init_app(global_conf, app_conf, package='ckan', paths=paths)
+    if six.PY2:
+        # Initialize Pylons own config object
+        pylons_config.init_app(
+            conf['global_conf'], conf, package='ckan', paths=paths)
 
-    # Update the main CKAN config object with the Pylons specific stuff, as it
-    # quite hard to keep them separated. This should be removed once Pylons
-    # support is dropped
-    config.update(pylons_config)
+        # Update the main CKAN config object with the Pylons specific stuff,
+        # as it is quite hard to keep them separated. This should be removed
+        # once Pylons support is dropped
+        config.update(pylons_config)
 
     # Setup the SQLAlchemy database engine
     # Suppress a couple of sqlalchemy warnings
@@ -162,6 +169,8 @@ def update_config():
     plugin might have changed the config values (for instance it might
     change ckan.site_url) '''
 
+    webassets_init()
+
     for plugin in p.PluginImplementations(p.IConfigurer):
         # must do update in place as this does not work:
         # config = plugin.update_config(config)
@@ -226,14 +235,22 @@ def update_config():
                              config.get('solr_password'))
     search.check_solr_schema_version()
 
-    routes_map = routing.make_map()
-    config['routes.map'] = routes_map
-    # The RoutesMiddleware needs its mapper updating if it exists
-    if 'routes.middleware' in config:
-        config['routes.middleware'].mapper = routes_map
-    # routes.named_routes is a CKAN thing
-    config['routes.named_routes'] = routing.named_routes
-    config['pylons.app_globals'] = app_globals.app_globals
+    if six.PY2:
+        routes_map = routing.make_map()
+
+    lib_plugins.reset_package_plugins()
+    lib_plugins.register_package_plugins()
+    lib_plugins.reset_group_plugins()
+    lib_plugins.register_group_plugins()
+
+    if six.PY2:
+        config['routes.map'] = routes_map
+        # The RoutesMiddleware needs its mapper updating if it exists
+        if 'routes.middleware' in config:
+            config['routes.middleware'].mapper = routes_map
+        # routes.named_routes is a CKAN thing
+        config['routes.named_routes'] = routing.named_routes
+        config['pylons.app_globals'] = app_globals.app_globals
 
     # initialise the globals
     app_globals.app_globals._init()
@@ -242,14 +259,14 @@ def update_config():
     config['pylons.h'] = helpers.helper_functions
 
     # Templates and CSS loading from configuration
-    valid_base_templates_folder_names = ['templates', 'templates-bs2']
+    valid_base_templates_folder_names = ['templates']
     templates = config.get('ckan.base_templates_folder', 'templates')
     config['ckan.base_templates_folder'] = templates
 
     if templates not in valid_base_templates_folder_names:
         raise CkanConfigurationException(
             'You provided an invalid value for ckan.base_templates_folder. '
-            'Possible values are: "templates" and "templates-bs2".'
+            'Possible values are: "templates".'
         )
 
     jinja2_templates_path = os.path.join(root, templates)
@@ -262,30 +279,30 @@ def update_config():
         template_paths = extra_template_paths.split(',') + template_paths
     config['computed_template_paths'] = template_paths
 
-    # Set the default language for validation messages from formencode
-    # to what is set as the default locale in the config
-    default_lang = config.get('ckan.locale_default', 'en')
-    formencode.api.set_stdtranslation(domain="FormEncode",
-                                      languages=[default_lang])
-
     # Markdown ignores the logger config, so to get rid of excessive
     # markdown debug messages in the log, set it to the level of the
     # root logger.
     logging.getLogger("MARKDOWN").setLevel(logging.getLogger().level)
 
-    # Create Jinja2 environment
-    env = jinja_extensions.Environment(
-        **jinja_extensions.get_jinja_env_options())
-    env.install_gettext_callables(_, ungettext, newstyle=True)
-    # custom filters
-    env.filters['empty_and_escape'] = jinja_extensions.empty_and_escape
-    config['pylons.app_globals'].jinja_env = env
+    if six.PY2:
+        # Create Jinja2 environment
+        env = jinja_extensions.Environment(
+            **jinja_extensions.get_jinja_env_options())
+        env.install_gettext_callables(_, ungettext, newstyle=True)
+        # custom filters
+        env.policies['ext.i18n.trimmed'] = True
+        env.filters['empty_and_escape'] = jinja_extensions.empty_and_escape
+        config['pylons.app_globals'].jinja_env = env
 
     # CONFIGURATION OPTIONS HERE (note: all config options will override
     # any Pylons config options)
 
+    # Enable pessimistic disconnect handling (added in SQLAlchemy 1.2)
+    # to eliminate database errors due to stale pooled connections
+    config.setdefault('sqlalchemy.pool_pre_ping', True)
+
     # Initialize SQLAlchemy
-    engine = sqlalchemy.engine_from_config(config, client_encoding='utf8')
+    engine = sqlalchemy.engine_from_config(config)
     model.init_model(engine)
 
     for plugin in p.PluginImplementations(p.IConfigurable):
@@ -311,6 +328,9 @@ def update_config():
     except sqlalchemy.exc.InternalError:
         # The database is not initialised.  Travis hits this
         pass
+    except sqlalchemy.exc.IntegrityError:
+        # Race condition, user already exists.
+        pass
 
     # Close current session and open database connections to ensure a clean
     # clean environment even if an error occurs later on
