@@ -22,9 +22,11 @@ of the revision history, rather than a feed of datasets.
 """
 # TODO fix imports
 import logging
-import urlparse
 
+import six
 from six import text_type
+from six.moves.urllib.parse import urlparse
+
 import webhelpers.feedgenerator
 
 import ckan.lib.base as base
@@ -131,7 +133,7 @@ def _create_atom_id(resource_path, authority_name=None, date_string=None):
         authority_name = config.get('ckan.feeds.authority_name', '').strip()
         if not authority_name:
             site_url = config.get('ckan.site_url', '').strip()
-            authority_name = urlparse.urlparse(site_url).netloc
+            authority_name = urlparse(site_url).netloc
 
     if not authority_name:
         log.warning('No authority_name available for feed generation.  '
@@ -401,8 +403,7 @@ class FeedController(base.BaseController):
 
             feed.add_item(
                 title=pkg.get('title', ''),
-                link=self.base_url + h.url_for(controller='package',
-                                               action='read',
+                link=self.base_url + h.url_for('dataset.read',
                                                id=pkg['id']),
                 description=pkg.get('notes', ''),
                 updated=h.date_str_to_datetime(pkg.get('metadata_modified')),
@@ -550,11 +551,11 @@ class _FixedAtom1Feed(webhelpers.feedgenerator.Atom1Feed):
 
         if(item['updated']):
             handler.addQuickElement(u'updated',
-                                    dfunc(item['updated']).decode('utf-8'))
+                                    six.ensure_text(dfunc(item['updated'])))
 
         if(item['published']):
             handler.addQuickElement(u'published',
-                                    dfunc(item['published']).decode('utf-8'))
+                                    six.ensure_text(dfunc(item['published'])))
 
     def add_root_elements(self, handler):
         """
