@@ -2,8 +2,9 @@
 
 import logging
 import datetime
-from urllib import urlencode
+from six.moves.urllib.parse import urlencode
 
+from collections import OrderedDict
 from pylons.i18n import get_lang
 from six import string_types, text_type
 
@@ -16,7 +17,7 @@ import ckan.model as model
 import ckan.authz as authz
 import ckan.lib.plugins
 import ckan.plugins as plugins
-from ckan.common import OrderedDict, c, config, request, _
+from ckan.common import c, config, request, _
 
 log = logging.getLogger(__name__)
 
@@ -220,11 +221,21 @@ class GroupController(base.BaseController):
             # Do not query for the group datasets when dictizing, as they will
             # be ignored and get requested on the controller anyway
             data_dict['include_datasets'] = False
+
+            # Do not query group members as they aren't used in the view
+            data_dict['include_users'] = False
+
             c.group_dict = self._action('group_show')(context, data_dict)
             c.group = context['group']
         except (NotFound, NotAuthorized):
             abort(404, _('Group not found'))
 
+        # if the user specified a group id, redirect to the group name
+        if data_dict['id'] == c.group_dict['id'] and \
+                data_dict['id'] != c.group_dict['name']:
+            h.redirect_to(controller=group_type, action='read',
+                          id=c.group_dict['name'])
+
         self._read(id, limit, group_type)
         return render(self._read_template(c.group_dict['type']),
                       extra_vars={'group_type': group_type})
@@ -797,7 +808,7 @@ class GroupController(base.BaseController):
                     revision_dict['timestamp'])
                 try:
                     dayHorizon = int(request.params.get('days'))
-                except:
+                except ValueError:
                     dayHorizon = 30
                 dayAge = (datetime.datetime.now() - revision_date).days
                 if dayAge >= dayHorizon:
@@ -861,6 +872,7 @@ class GroupController(base.BaseController):
             group_dict = get_action('group_show')(context, data_dict)
             h.flash_success(_("You are now following {0}").format(
                 group_dict['title']))
+            id = group_dict['name']
         except ValidationError as e:
             error_message = (e.message or e.error_summary
                              or e.error_dict)
@@ -881,6 +893,7 @@ class GroupController(base.BaseController):
             group_dict = get_action('group_show')(context, data_dict)
             h.flash_success(_("You are no longer following {0}").format(
                 group_dict['title']))
+            id = group_dict['name']
         except ValidationError as e:
             error_message = (e.message or e.error_summary
                              or e.error_dict)
