@@ -1,13 +1,13 @@
 # encoding: utf-8
 
 import logging
-from urllib import urlencode
+import six
+from six.moves.urllib.parse import urlencode
 import datetime
 import mimetypes
 import cgi
 
-from ckan.common import config
-from paste.deploy.converters import asbool
+from collections import OrderedDict
 import paste.fileapp
 from six import string_types, text_type
 
@@ -24,8 +24,8 @@ import ckan.lib.uploader as uploader
 import ckan.plugins as p
 import ckan.lib.render
 
-from ckan.common import OrderedDict, _, json, request, c, response
-from home import CACHE_PARAMETERS
+from ckan.common import config, asbool, _, json, request, c, response
+from ckan.controllers.home import CACHE_PARAMETERS
 
 log = logging.getLogger(__name__)
 
@@ -230,7 +230,7 @@ class PackageController(base.BaseController):
             # types any search page. Potential alternatives are do show them
             # on the default search page (dataset) or on one other search page
             search_all_type = config.get(
-                                  'ckan.search.show_all_types', 'dataset')
+                'ckan.search.show_all_types', 'dataset')
             search_all = False
 
             try:
@@ -258,7 +258,7 @@ class PackageController(base.BaseController):
                 'tags': _('Tags'),
                 'res_format': _('Formats'),
                 'license_id': _('Licenses'),
-                }
+            }
 
             for facet in h.facets():
                 if facet in default_facet_titles:
@@ -393,14 +393,13 @@ class PackageController(base.BaseController):
 
         # can the resources be previewed?
         for resource in c.pkg_dict['resources']:
-            # Backwards compatibility with preview interface
-            resource['can_be_previewed'] = self._resource_preview(
-                {'resource': resource, 'package': c.pkg_dict})
-
             resource_views = get_action('resource_view_list')(
                 context, {'id': resource['id']})
             resource['has_views'] = len(resource_views) > 0
 
+            # Backwards compatibility with preview interface
+            resource['can_be_previewed'] = bool(len(resource_views))
+
         package_type = c.pkg_dict['type'] or 'dataset'
         self._setup_template_variables(context, {'id': id},
                                        package_type=package_type)
@@ -472,7 +471,7 @@ class PackageController(base.BaseController):
                     revision_dict['timestamp'])
                 try:
                     dayHorizon = int(request.params.get('days'))
-                except:
+                except ValueError:
                     dayHorizon = 30
                 dayAge = (datetime.datetime.now() - revision_date).days
                 if dayAge >= dayHorizon:
@@ -649,7 +648,7 @@ class PackageController(base.BaseController):
 
             # see if we have any data that we are trying to save
             data_provided = False
-            for key, value in data.iteritems():
+            for key, value in six.iteritems(data):
                 if ((value or isinstance(value, cgi.FieldStorage))
                         and key != 'resource_type'):
                     data_provided = True
@@ -667,8 +666,12 @@ class PackageController(base.BaseController):
                 except NotFound:
                     abort(404, _('The dataset {id} could not be found.'
                                  ).format(id=id))
-                if not len(data_dict['resources']):
-                    # no data so keep on page
+                require_resources = asbool(
+                    config.get('ckan.dataset.create_on_ui_requires_resources',
+                               'True')
+                )
+                if require_resources and not len(data_dict['resources']):
+                    # no data and configured to require resource: stay on page
                     msg = _('You must add at least one data resource')
                     # On new templates do not use flash message
 
@@ -698,11 +701,6 @@ class PackageController(base.BaseController):
             except ValidationError as e:
                 errors = e.error_dict
                 error_summary = e.error_summary
-                if data.get('url_type') == 'upload' and data.get('url'):
-                    data['url'] = ''
-                    data['url_type'] = ''
-                    data['previous_upload'] = True
-
                 return self.new_resource(id, data, errors, error_summary)
             except NotAuthorized:
                 abort(403, _('Unauthorized to create a resource'))
@@ -1117,13 +1115,12 @@ class PackageController(base.BaseController):
         c.datastore_api = '%s/api/action' % \
             config.get('ckan.site_url', '').rstrip('/')
 
-        c.resource['can_be_previewed'] = self._resource_preview(
-            {'resource': c.resource, 'package': c.package})
-
         resource_views = get_action('resource_view_list')(
             context, {'id': resource_id})
         c.resource['has_views'] = len(resource_views) > 0
 
+        c.resource['can_be_previewed'] = bool(len(resource_views))
+
         current_resource_view = None
         view_id = request.GET.get('view_id')
         if c.resource['can_be_previewed'] and not view_id:
