@@ -9,36 +9,34 @@ import time
 import inspect
 import sys
 
-from pylons import cache
-from pylons.controllers import WSGIController
-from pylons.controllers.util import abort as _abort
-from pylons.decorators import jsonify
-from pylons.templating import cached_template, pylons_globals
-from webhelpers.html import literal
+from jinja2.exceptions import TemplateNotFound
 
+import six
 from flask import (
     render_template as flask_render_template,
     abort as flask_abort
 )
-import ckan.exceptions
-import ckan
+
 import ckan.lib.i18n as i18n
 import ckan.lib.render as render_
 import ckan.lib.helpers as h
 import ckan.lib.app_globals as app_globals
 import ckan.plugins as p
 import ckan.model as model
-
 from ckan.views import (identify_user,
                         set_cors_headers_for_response,
                         check_session_cookie,
                         )
+from ckan.common import (c, request, config,
+                         session, is_flask_request, asbool)
+
+
+if six.PY2:
+    from pylons.controllers import WSGIController
+    from pylons.controllers.util import abort as _abort
+    from pylons.templating import cached_template, pylons_globals
+    from ckan.common import response
 
-# These imports are for legacy usages and will be removed soon these should
-# be imported directly from ckan.common for internal ckan code and via the
-# plugins.toolkit for extensions.
-from ckan.common import (json, _, ungettext, c, request, response, config,
-                         session, is_flask_request)
 
 log = logging.getLogger(__name__)
 
@@ -77,17 +75,40 @@ def abort(status_code=None, detail='', headers=None, comment=None):
                   comment=comment)
 
 
-def render_snippet(template_name, **kw):
+def render_snippet(*template_names, **kw):
     ''' Helper function for rendering snippets. Rendered html has
     comment tags added to show the template used. NOTE: unlike other
     render functions this takes a list of keywords instead of a dict for
-    the extra template variables. '''
+    the extra template variables.
 
-    output = render(template_name, extra_vars=kw)
-    if config.get('debug'):
-        output = ('\n<!-- Snippet %s start -->\n%s\n<!-- Snippet %s end -->\n'
-                  % (template_name, output, template_name))
-    return literal(output)
+    :param template_names: the template to render, optionally with fallback
+        values, for when the template can't be found. For each, specify the
+        relative path to the template inside the registered tpl_dir.
+    :type template_names: str
+    :param kw: extra template variables to supply to the template
+    :type kw: named arguments of any type that are supported by the template
+    '''
+
+    last_exc = None
+    for template_name in template_names:
+        try:
+            output = render(template_name, extra_vars=kw)
+            if asbool(config.get('debug')):
+                output = (
+                    '\n<!-- Snippet %s start -->\n%s\n<!-- Snippet %s end -->'
+                    '\n' % (template_name, output, template_name))
+            return h.literal(output)
+        except TemplateNotFound as exc:
+            if exc.name == template_name:
+                # the specified template doesn't exist - try the next
+                # fallback, but store the exception in case it was
+                # last one
+                last_exc = exc
+                continue
+            # a nested template doesn't exist - don't fallback
+            raise exc
+    else:
+        raise last_exc or TemplateNotFound
 
 
 def render_jinja2(template_name, extra_vars):
@@ -126,6 +147,7 @@ def render(template_name, extra_vars=None, *pargs, **kwargs):
                                             *pargs, **kwargs)
         return cached_template(template_name, renderer)
 
+    _allow_caching()
     return flask_render_template(template_name, **extra_vars)
 
 
@@ -196,14 +218,10 @@ def _pylons_prepare_renderer(template_name, extra_vars, cache_key=None,
     # Don't cache if we have set the __no_cache__ param in the query string.
     elif request.params.get('__no_cache__'):
         allow_cache = False
-    # Don't cache if we have extra vars containing data.
-    elif extra_vars:
-        for k, v in extra_vars.iteritems():
-            allow_cache = False
-            break
+    # Don't cache if caching is not enabled in config
+    elif not asbool(config.get('ckan.cache_enabled', False)):
+        allow_cache = False
 
-    # TODO: replicate this logic in Flask once we start looking at the
-    # rendering for the frontend controllers
     set_pylons_response_headers(allow_cache)
 
     if not allow_cache:
@@ -213,49 +231,83 @@ def _pylons_prepare_renderer(template_name, extra_vars, cache_key=None,
     return render_template
 
 
+def _allow_caching(cache_force=None):
+    # Caching Logic
+
+    allow_cache = True
+    # Force cache or not if explicit.
+    if cache_force is not None:
+        allow_cache = cache_force
+    # Do not allow caching of pages for logged in users/flash messages etc.
+    elif ('user' in c and c.user) or _is_valid_session_cookie_data():
+        allow_cache = False
+    # Tests etc.
+    elif 'REMOTE_USER' in request.environ:
+        allow_cache = False
+    # Don't cache if based on a non-cachable template used in this.
+    elif request.environ.get('__no_cache__'):
+        allow_cache = False
+    # Don't cache if we have set the __no_cache__ param in the query string.
+    elif request.params.get('__no_cache__'):
+        allow_cache = False
+    # Don't cache if caching is not enabled in config
+    elif not asbool(config.get('ckan.cache_enabled', False)):
+        allow_cache = False
+
+    if not allow_cache:
+        # Prevent any further rendering from being cached.
+        request.environ['__no_cache__'] = True
+
+
+def _is_valid_session_cookie_data():
+    is_valid_cookie_data = False
+    for key, value in session.items():
+        if not key.startswith(u'_') and value:
+            is_valid_cookie_data = True
+            break
+
+    return is_valid_cookie_data
+
+
 class ValidationException(Exception):
     pass
 
 
-class BaseController(WSGIController):
-    '''Base class for CKAN controller classes to inherit from.
+if six.PY2:
+    class BaseController(WSGIController):
+        '''Base class for CKAN controller classes to inherit from.
 
-    '''
-    repo = model.repo
-    log = logging.getLogger(__name__)
+        '''
+        repo = model.repo
+        log = logging.getLogger(__name__)
 
-    def __before__(self, action, **params):
-        c.__timer = time.time()
-        app_globals.app_globals._check_uptodate()
+        def __before__(self, action, **params):
+            c.__timer = time.time()
+            app_globals.app_globals._check_uptodate()
 
-        identify_user()
+            identify_user()
 
-        i18n.handle_request(request, c)
+            i18n.handle_request(request, c)
 
-    def __call__(self, environ, start_response):
-        """Invoke the Controller"""
-        # WSGIController.__call__ dispatches to the Controller method
-        # the request is routed to. This routing information is
-        # available in environ['pylons.routes_dict']
+        def __call__(self, environ, start_response):
+            """Invoke the Controller"""
+            # WSGIController.__call__ dispatches to the Controller method
+            # the request is routed to. This routing information is
+            # available in environ['pylons.routes_dict']
 
-        try:
-            res = WSGIController.__call__(self, environ, start_response)
-        finally:
-            model.Session.remove()
+            try:
+                res = WSGIController.__call__(self, environ, start_response)
+            finally:
+                model.Session.remove()
 
-        check_session_cookie(response)
+            check_session_cookie(response)
 
-        return res
+            return res
 
-    def __after__(self, action, **params):
+        def __after__(self, action, **params):
 
-        set_cors_headers_for_response(response)
+            set_cors_headers_for_response(response)
 
-        r_time = time.time() - c.__timer
-        url = request.environ['CKAN_CURRENT_URL'].split('?')[0]
-        log.info(' %s render time %.3f seconds' % (url, r_time))
-
-
-# Include the '_' function in the public names
-__all__ = [__name for __name in locals().keys() if not __name.startswith('_')
-           or __name == '_']
+            r_time = time.time() - c.__timer
+            url = request.environ['CKAN_CURRENT_URL'].split('?')[0]
+            log.info(' %s render time %.3f seconds' % (url, r_time))
