@@ -1,10 +1,11 @@
 # encoding: utf-8
 
-from ckan.common import config
+from ckan.common import config, asint
+
+import requests
+
+TIMEOUT = asint(config.get('ckan.requests.timeout', 5))
 
-import urllib
-import urllib2
-import json
 
 def check_recaptcha(request):
     '''Check a user\'s recaptcha submission is valid, and raise CaptchaError
@@ -14,7 +15,8 @@ def check_recaptcha(request):
         # Recaptcha not enabled
         return
 
-    client_ip_address = request.environ.get('REMOTE_ADDR', 'Unknown IP Address')
+    client_ip_address = request.environ.get(
+        'REMOTE_ADDR', 'Unknown IP Address')
 
     # reCAPTCHA v2
     recaptcha_response_field = request.form.get('g-recaptcha-response', '')
@@ -23,12 +25,13 @@ def check_recaptcha(request):
     # recaptcha_response_field will be unicode if there are foreign chars in
     # the user input. So we need to encode it as utf8 before urlencoding or
     # we get an exception (#1431).
-    params = urllib.urlencode(dict(secret=recaptcha_private_key,
-                                   remoteip=client_ip_address,
-                                   response=recaptcha_response_field.encode('utf8')))
-    f = urllib2.urlopen(recaptcha_server_name, params)
-    data = json.load(f)
-    f.close()
+    params = dict(
+        secret=recaptcha_private_key,
+        remoteip=client_ip_address,
+        response=recaptcha_response_field.encode('utf8')
+    )
+    response = requests.get(recaptcha_server_name, params, timeout=TIMEOUT)
+    data = response.json()
 
     try:
         if not data['success']:
@@ -37,5 +40,6 @@ def check_recaptcha(request):
         # Something weird with recaptcha response
         raise CaptchaError()
 
+
 class CaptchaError(ValueError):
     pass
