@@ -1,15 +1,16 @@
 # encoding: utf-8
 
 import logging
-from collections import defaultdict
 import datetime
 
 from six import string_types, text_type
 
 import ckan.model as model
+import ckan.logic as logic
 
 log = logging.getLogger(__name__)
 
+
 class CreateTestData(object):
     # keep track of the objects created by this class so that
     # tests can easy call delete() method to delete them all again.
@@ -61,23 +62,19 @@ class CreateTestData(object):
         cls.user_refs.append(u'tester')
 
     @classmethod
-
     def create_translations_test_data(cls):
-        import ckan.model
-        CreateTestData.create()
-        rev = ckan.model.repo.new_revision()
-        rev.author = CreateTestData.author
-        rev.message = u'Creating test translations.'
-
-        sysadmin_user = ckan.model.User.get('testsysadmin')
-        package = ckan.model.Package.get('annakarenina')
+        package = model.Package.get('annakarenina')
+        if not package:
+            CreateTestData.create()
+            package = model.Package.get('annakarenina')
 
+        sysadmin_user = model.User.get('testsysadmin')
         # Add some new tags to the package.
         # These tags are codes that are meant to be always translated before
         # display, if not into the user's current language then into the
         # fallback language.
-        package.add_tags([ckan.model.Tag('123'), ckan.model.Tag('456'),
-                ckan.model.Tag('789')])
+        package.add_tags([model.Tag('123'), model.Tag('456'),
+                model.Tag('789')])
 
         # Add the above translations to CKAN.
         for (lang_code, translations) in (('de', german_translations),
@@ -90,26 +87,29 @@ class CreateTestData(object):
                             'lang_code': lang_code,
                             }
                     context = {
-                        'model': ckan.model,
-                        'session': ckan.model.Session,
+                        'model': model,
+                        'session': model.Session,
                         'user': sysadmin_user.name,
                     }
-                    ckan.logic.action.update.term_translation_update(context,
+                    logic.action.update.term_translation_update(context,
                             data_dict)
 
-        ckan.model.Session.commit()
+        model.Session.commit()
 
+    @classmethod
     def create_vocabs_test_data(cls):
-        import ckan.model
-        CreateTestData.create()
-        sysadmin_user = ckan.model.User.get('testsysadmin')
-        annakarenina = ckan.model.Package.get('annakarenina')
-        warandpeace = ckan.model.Package.get('warandpeace')
+        warandpeace = model.Package.get('warandpeace')
+        if not warandpeace:
+            CreateTestData.create()
+            warandpeace = model.Package.get('warandpeace')
+        
+        sysadmin_user = model.User.get('testsysadmin')
+        annakarenina = model.Package.get('annakarenina')
 
         # Create a couple of vocabularies.
         context = {
-                'model': ckan.model,
-                'session': ckan.model.Session,
+                'model': model,
+                'session': model.Session,
                 'user': sysadmin_user.name
                 }
         data_dict = {
@@ -117,18 +117,18 @@ class CreateTestData(object):
                 'tags': [{'name': 'Drama'}, {'name': 'Sci-Fi'},
                     {'name': 'Mystery'}],
                 }
-        ckan.logic.action.create.vocabulary_create(context, data_dict)
+        logic.action.create.vocabulary_create(context, data_dict)
 
         data_dict = {
                 'name': 'Actors',
                 'tags': [{'name': 'keira-knightley'}, {'name': 'jude-law'},
                     {'name': 'alessio-boni'}],
                 }
-        ckan.logic.action.create.vocabulary_create(context, data_dict)
+        logic.action.create.vocabulary_create(context, data_dict)
 
         # Add some vocab tags to some packages.
-        genre_vocab = ckan.model.Vocabulary.get('Genre')
-        actors_vocab = ckan.model.Vocabulary.get('Actors')
+        genre_vocab = model.Vocabulary.get('Genre')
+        actors_vocab = model.Vocabulary.get('Actors')
         annakarenina.add_tag_by_name('Drama', vocab=genre_vocab)
         annakarenina.add_tag_by_name('keira-knightley', vocab=actors_vocab)
         annakarenina.add_tag_by_name('jude-law', vocab=actors_vocab)
@@ -158,12 +158,9 @@ class CreateTestData(object):
             if isinstance(package_dicts, dict):
                 package_dicts = [package_dicts]
             for item in package_dicts:
-                rev = model.repo.new_revision()
-                rev.author = cls.author
-                rev.message = u'Creating test packages.'
                 pkg_dict = {}
                 for field in cls.pkg_core_fields:
-                    if item.has_key(field):
+                    if field in item:
                         pkg_dict[field] = text_type(item[field])
                 if model.Package.by_name(pkg_dict['name']):
                     log.warning('Cannot create package "%s" as it already exists.' % \
@@ -254,7 +251,6 @@ class CreateTestData(object):
 
         needs_commit = False
 
-        rev = model.repo.new_revision()
         for group_name in extra_group_names:
             group = model.Group(name=text_type(group_name))
             model.Session.add(group)
@@ -288,10 +284,6 @@ class CreateTestData(object):
             needs_commit = False
 
         if relationships:
-            rev = model.repo.new_revision()
-            rev.author = cls.author
-            rev.message = u'Creating package relationships.'
-
             def pkg(pkg_name):
                 return model.Package.by_name(text_type(pkg_name))
             for subject_name, relationship, object_name in relationships:
@@ -307,8 +299,6 @@ class CreateTestData(object):
         '''A more featured interface for creating groups.
         All group fields can be filled, packages added, can have
         an admin user and be a member of other groups.'''
-        rev = model.repo.new_revision()
-        rev.author = cls.author
         if admin_user_name:
             admin_users = [model.User.by_name(admin_user_name)]
         else:
@@ -356,8 +346,6 @@ class CreateTestData(object):
             # 2. The next Group created may have this Group as a parent so
             #    creation of the Member needs to refer to this one.
             model.Session.commit()
-            rev = model.repo.new_revision()
-            rev.author = cls.author
             # add it to a parent's group
             if 'parent' in group_dict:
                 parent = model.Group.by_name(text_type(group_dict['parent']))
@@ -371,14 +359,6 @@ class CreateTestData(object):
     @classmethod
     def create(cls, auth_profile="", package_type=None):
         model.Session.remove()
-        rev = model.repo.new_revision()
-        # same name as user we create below
-        rev.author = cls.author
-        rev.message = u'''Creating test data.
- * Package: annakarenina
- * Package: warandpeace
- * Associated tags, etc etc
-'''
         if auth_profile == "publisher":
             organization_group = model.Group(name=u"organization_group",
                                              type="organization")
@@ -489,6 +469,12 @@ left arrow <
             sysadmin,
             ])
         cls.user_refs.extend([u'tester', u'joeadmin', u'annafan', u'russianfan', u'testsysadmin'])
+
+        # Create activities for packages
+        for item in [pkg1, pkg2]:
+            activity = item.activity_stream_item('new', 'not logged in')
+            model.Session.add(activity)
+
         model.repo.commit_and_remove()
 
     # method used in DGU and all good tests elsewhere
@@ -512,9 +498,12 @@ left arrow <
         user_ref = name
         assert user_ref
         for k, v in user_dict.items():
-            if v:
-                # avoid unicode warnings
-                user_dict[k] = text_type(v)
+            if v is not None:
+                if bool(v):
+                    user_dict[k] = v
+                else:
+                    # avoid unicode warnings
+                    user_dict[k] = text_type(v)
         user = model.User(name=text_type(name), **user_dict)
         model.Session.add(user)
         cls.user_refs.append(user_ref)
@@ -584,7 +573,6 @@ left arrow <
 
     @classmethod
     def make_some_vocab_tags(cls):
-        model.repo.new_revision()
 
         # Create a couple of vocabularies.
         genre_vocab = model.Vocabulary(u'genre')
