@@ -3,20 +3,18 @@
 import datetime
 import uuid
 import logging
-import six
 
 from sqlalchemy.orm import class_mapper
+import six
 from six import string_types
 
 import ckan.lib.dictization as d
-import ckan.lib.helpers as h
 import ckan.authz as authz
 
 log = logging.getLogger(__name__)
 
 
 def resource_dict_save(res_dict, context):
-
     model = context["model"]
     session = context["session"]
 
@@ -37,44 +35,39 @@ def resource_dict_save(res_dict, context):
     if res_dict.get('url') and res_dict.get('url_type') == u'upload':
         res_dict['url'] = res_dict['url'].rsplit('/')[-1]
 
-    # Resource extras not submitted will be removed from the existing extras
-    # dict
-    new_extras = {}
-    has_changed = False
-    for key, value in six.iteritems(res_dict):
-        if key in ('extras', 'revision_timestamp', 'tracking_summary'):
-            continue
-        if key in fields:
-            if isinstance(getattr(obj, key), datetime.datetime):
-                if isinstance(value, string_types):
-                    db_value = getattr(obj, key).isoformat()
-                else:
-                    db_value = getattr(obj, key)
-                if  db_value == value:
-                    continue
-                if key == 'last_modified' and not new:
-                    obj.url_changed = True
-            if key == 'url' and not new and obj.url != value:
-                obj.url_changed = True
-            setattr(obj, key, value)
-        else:
-            # resources save extras directly onto the object, instead
-            # of in a separate extras field like packages and groups
-            new_extras[key] = value
+    # unconditionally ignored fields
+    res_dict.pop('extras', None)
+    res_dict.pop('revision_timestamp', None)
+    res_dict.pop('tracking_summary', None)
 
+    changed, skipped = obj.from_dict(res_dict)
+
+    if 'url' in changed or ('last_modified' in changed and not new):
+        obj.url_changed = True
+
+    if changed or obj.extras != skipped:
+        obj.metadata_modified = datetime.datetime.utcnow()
     obj.state = u'active'
-    obj.extras = new_extras
+    obj.extras = skipped
 
     session.add(obj)
     return obj
 
+
 def package_resource_list_save(res_dicts, package, context):
     allow_partial_update = context.get("allow_partial_update", False)
     if res_dicts is None and allow_partial_update:
         return
 
+    session = context['session']
+    model = context['model']
     resource_list = package.resources_all
-    old_list = package.resources_all[:]
+    old_list = session.query(model.Resource) \
+        .filter(model.Resource.package_id == package.id) \
+        .filter(model.Resource.state != 'deleted')[:]
+    deleted_list = session.query(model.Resource) \
+        .filter(model.Resource.package_id == package.id) \
+        .filter(model.Resource.state == 'deleted')[:]
 
     obj_list = []
     for res_dict in res_dicts or []:
@@ -94,22 +87,23 @@ def package_resource_list_save(res_dicts, package, context):
     # according to their ordering in the obj_list.
     resource_list[:] = obj_list
 
+    # Permanently remove old deleted resources
+    for resource in set(deleted_list) - set(obj_list):
+        resource.purge()
     # Mark any left-over resources as deleted
     for resource in set(old_list) - set(obj_list):
         resource.state = 'deleted'
         resource_list.append(resource)
 
 
-def package_extras_save(extra_dicts, obj, context):
+def package_extras_save(extra_dicts, pkg, context):
     allow_partial_update = context.get("allow_partial_update", False)
     if extra_dicts is None and allow_partial_update:
         return
 
-    model = context["model"]
     session = context["session"]
 
-    extras_list = obj.extras_list
-    old_extras = dict((extra.key, extra) for extra in extras_list)
+    old_extras = pkg._extras
 
     new_extras = {}
     for extra_dict in extra_dicts or []:
@@ -120,28 +114,22 @@ def package_extras_save(extra_dicts, obj, context):
             pass
         else:
             new_extras[extra_dict["key"]] = extra_dict["value"]
+
     #new
     for key in set(new_extras.keys()) - set(old_extras.keys()):
-        state = 'active'
-        extra = model.PackageExtra(state=state, key=key, value=new_extras[key])
-        session.add(extra)
-        extras_list.append(extra)
+        pkg.extras[key] = new_extras[key]
     #changed
     for key in set(new_extras.keys()) & set(old_extras.keys()):
         extra = old_extras[key]
-        if new_extras[key] == extra.value and extra.state != 'deleted':
+        if new_extras[key] == extra.value:
             continue
-        state = 'active'
         extra.value = new_extras[key]
-        extra.state = state
         session.add(extra)
     #deleted
     for key in set(old_extras.keys()) - set(new_extras.keys()):
         extra = old_extras[key]
-        if extra.state == 'deleted':
-            continue
-        state = 'deleted'
-        extra.state = state
+        session.delete(extra)
+
 
 def package_tag_list_save(tag_dicts, package, context):
     allow_partial_update = context.get("allow_partial_update", False)
@@ -309,7 +297,7 @@ def package_dict_save(pkg_dict, context):
         objects = pkg_dict.get('relationships_as_object')
         relationship_list_save(objects, pkg, 'relationships_as_object', context)
 
-    extras = package_extras_save(pkg_dict.get("extras"), pkg, context)
+    package_extras_save(pkg_dict.get("extras"), pkg, context)
 
     return pkg
 
@@ -427,7 +415,7 @@ def group_dict_save(group_dict, context, prevent_packages_update=False):
     package_ids.extend( pkgs_edited['added'] )
     if package_ids:
         session.commit()
-        map( rebuild, package_ids )
+        [rebuild(package_id) for package_id in package_ids]
 
     return group
 
@@ -445,7 +433,12 @@ def user_dict_save(user_dict, context):
     if 'password' in user_dict and not len(user_dict['password']):
         del user_dict['password']
 
-    user = d.table_dict_save(user_dict, User, context)
+    user = d.table_dict_save(
+        user_dict,
+        User,
+        context,
+        extra_attrs=['_password'],  # for setting password_hash directly
+    )
 
     return user
 
@@ -458,7 +451,7 @@ def package_api_to_dict(api1_dict, context):
 
     dictized = {}
 
-    for key, value in api1_dict.iteritems():
+    for key, value in six.iteritems(api1_dict):
         new_value = value
         if key == 'tags':
             if isinstance(value, string_types):
@@ -473,7 +466,7 @@ def package_api_to_dict(api1_dict, context):
 
             new_value = []
 
-            for extras_key, extras_value in updated_extras.iteritems():
+            for extras_key, extras_value in six.iteritems(updated_extras):
                 new_value.append({"key": extras_key,
                                   "value": extras_value})
 
@@ -497,7 +490,7 @@ def group_api_to_dict(api1_dict, context):
 
     dictized = {}
 
-    for key, value in api1_dict.iteritems():
+    for key, value in six.iteritems(api1_dict):
         new_value = value
         if key == 'packages':
             new_value = [{"id": item} for item in value]
@@ -524,13 +517,12 @@ def activity_dict_save(activity_dict, context):
     session = context['session']
     user_id = activity_dict['user_id']
     object_id = activity_dict['object_id']
-    revision_id = activity_dict['revision_id']
     activity_type = activity_dict['activity_type']
-    if activity_dict.has_key('data'):
+    if 'data' in activity_dict:
         data = activity_dict['data']
     else:
         data = None
-    activity_obj = model.Activity(user_id, object_id, revision_id,
+    activity_obj = model.Activity(user_id, object_id,
             activity_type, data)
     session.add(activity_obj)
 
@@ -563,7 +555,7 @@ def vocabulary_dict_save(vocabulary_dict, context):
     vocabulary_obj = model.Vocabulary(vocabulary_name)
     session.add(vocabulary_obj)
 
-    if vocabulary_dict.has_key('tags'):
+    if 'tags' in vocabulary_dict:
         vocabulary_tag_list_save(vocabulary_dict['tags'], vocabulary_obj,
             context)
 
@@ -576,10 +568,10 @@ def vocabulary_dict_update(vocabulary_dict, context):
 
     vocabulary_obj = model.vocabulary.Vocabulary.get(vocabulary_dict['id'])
 
-    if vocabulary_dict.has_key('name'):
+    if 'name' in vocabulary_dict:
         vocabulary_obj.name = vocabulary_dict['name']
 
-    if vocabulary_dict.has_key('tags'):
+    if 'tags' in vocabulary_dict:
         vocabulary_tag_list_save(vocabulary_dict['tags'], vocabulary_obj,
             context)
 
@@ -609,10 +601,21 @@ def resource_view_dict_save(data_dict, context):
     if resource_view:
         data_dict['id'] = resource_view.id
     config = {}
-    for key, value in data_dict.iteritems():
+    for key, value in six.iteritems(data_dict):
         if key not in model.ResourceView.get_columns():
             config[key]  = value
     data_dict['config'] = config
 
 
     return d.table_dict_save(data_dict, model.ResourceView, context)
+
+
+def api_token_save(data_dict, context):
+    model = context[u"model"]
+    return d.table_dict_save(
+        {
+            u"user_id": model.User.get(data_dict['user']).id,
+            u"name": data_dict[u"name"]
+        },
+        model.ApiToken, context
+    )
