@@ -1,3 +1,5 @@
+# encoding: utf-8
+
 '''
 Code for generating email notifications for users (e.g. email notifications for
 new activities in your dashboard activity stream) and emailing them to the
@@ -7,13 +9,11 @@ users.
 import datetime
 import re
 
-import pylons
-
 import ckan.model as model
 import ckan.logic as logic
 import ckan.lib.base as base
 
-from ckan.common import ungettext
+from ckan.common import ungettext, config
 
 
 def string_to_timedelta(s):
@@ -98,10 +98,10 @@ def _notifications_for_activities(activities, user_dict):
     # certain types of activity to be sent in their own individual emails,
     # etc.
     subject = ungettext(
-        "1 new activity from {site_title}",
+        "{n} new activity from {site_title}",
         "{n} new activities from {site_title}",
         len(activities)).format(
-                site_title=pylons.config.get('ckan.site_title'),
+                site_title=config.get('ckan.site_title'),
                 n=len(activities))
     body = base.render(
             'activity_streams/activity_stream_email_notifications.text',
@@ -188,11 +188,11 @@ def get_and_send_notifications_for_user(user):
 
     # Parse the email_notifications_since config setting, email notifications
     # from longer ago than this time will not be sent.
-    email_notifications_since = pylons.config.get(
+    email_notifications_since = config.get(
             'ckan.email_notifications_since', '2 days')
     email_notifications_since = string_to_timedelta(
             email_notifications_since)
-    email_notifications_since = (datetime.datetime.now()
+    email_notifications_since = (datetime.datetime.utcnow()
             - email_notifications_since)
 
     # FIXME: We are accessing model from lib here but I'm not sure what
@@ -206,7 +206,6 @@ def get_and_send_notifications_for_user(user):
             activity_stream_last_viewed)
 
     notifications = get_notifications(user, since)
-
     # TODO: Handle failures from send_email_notification.
     for notification in notifications:
         send_notification(user, notification)
@@ -215,13 +214,13 @@ def get_and_send_notifications_for_user(user):
     # else to do unless we add a update_email_last_sent()
     # logic function which would only be needed by this lib.
     dash = model.Dashboard.get(user['id'])
-    dash.email_last_sent = datetime.datetime.now()
+    dash.email_last_sent = datetime.datetime.utcnow()
     model.repo.commit()
 
 
 def get_and_send_notifications_for_all_users():
     context = {'model': model, 'session': model.Session, 'ignore_auth': True,
-            'keep_sensitive_data': True}
+            'keep_email': True}
     users = logic.get_action('user_list')(context, {})
     for user in users:
         get_and_send_notifications_for_user(user)
