@@ -1,14 +1,11 @@
 # encoding: utf-8
 
 import copy
-import formencode as fe
-import inspect
 import json
 
+import six
 from six import text_type
-from ckan.common import config
-
-from ckan.common import _
+from ckan.common import config, _
 
 
 class Missing(object):
@@ -36,8 +33,8 @@ class Missing(object):
     def __hex__(self):
         raise Invalid(_('Missing value'))
 
-    def __nonzero__(self):
-        return False
+    def __len__(self):
+        return 0
 
 
 missing = Missing()
@@ -49,7 +46,7 @@ class State(object):
 
 class DictizationError(Exception):
     def __str__(self):
-        return text_type(self).encode('utf8')
+        return six.ensure_str(self.__unicode__())
 
     def __unicode__(self):
         if hasattr(self, 'error') and self.error:
@@ -109,7 +106,7 @@ def flatten_schema(schema, flattened=None, key=None):
     flattened = flattened or {}
     old_key = key or []
 
-    for key, value in schema.iteritems():
+    for key, value in six.iteritems(schema):
         new_key = old_key + [key]
         if isinstance(value, dict):
             flattened = flatten_schema(value, flattened, new_key)
@@ -124,7 +121,7 @@ def get_all_key_combinations(data, flattened_schema):
     match the schema ignoring the last value in the tuple.
 
     '''
-    schema_prefixes = set([key[:-1] for key in flattened_schema])
+    schema_prefixes = {key[:-1] for key in flattened_schema}
     combinations = set([()])
 
     for key in sorted(data.keys(), key=flattened_order_key):
@@ -156,7 +153,7 @@ def make_full_schema(data, schema):
         for key in combination[::2]:
             sub_schema = sub_schema[key]
 
-        for key, value in sub_schema.iteritems():
+        for key, value in six.iteritems(sub_schema):
             if isinstance(value, list):
                 full_schema[combination + (key,)] = value
 
@@ -179,8 +176,10 @@ def augment_data(data, schema):
 
     new_data = copy.copy(data)
 
+    keys_to_remove = []
+    junk = {}
+    extras_keys = {}
     # fill junk and extras
-
     for key, value in new_data.items():
         if key in full_schema:
             continue
@@ -193,16 +192,21 @@ def augment_data(data, schema):
                 raise DataError('Only lists of dicts can be placed against '
                                 'subschema %s, not %s' %
                                 (key, type(data[key])))
-
         if key[:-1] in key_combinations:
             extras_key = key[:-1] + ('__extras',)
-            extras = new_data.get(extras_key, {})
+            extras = extras_keys.get(extras_key, {})
             extras[key[-1]] = value
-            new_data[extras_key] = extras
+            extras_keys[extras_key] = extras
         else:
-            junk = new_data.get(("__junk",), {})
             junk[key] = value
-            new_data[("__junk",)] = junk
+        keys_to_remove.append(key)
+
+    if junk:
+        new_data[("__junk",)] = junk
+    for extra_key in extras_keys:
+        new_data[extra_key] = extras_keys[extra_key]
+
+    for key in keys_to_remove:
         new_data.pop(key)
 
     # add missing
@@ -216,22 +220,6 @@ def augment_data(data, schema):
 
 def convert(converter, key, converted_data, errors, context):
 
-    if inspect.isclass(converter) and issubclass(converter, fe.Validator):
-        try:
-            value = converted_data.get(key)
-            value = converter().to_python(value, state=context)
-        except fe.Invalid as e:
-            errors[key].append(e.msg)
-        return
-
-    if isinstance(converter, fe.Validator):
-        try:
-            value = converted_data.get(key)
-            value = converter.to_python(value, state=context)
-        except fe.Invalid as e:
-            errors[key].append(e.msg)
-        return
-
     try:
         value = converter(converted_data.get(key))
         converted_data[key] = value
@@ -268,7 +256,7 @@ def convert(converter, key, converted_data, errors, context):
 
 def _remove_blank_keys(schema):
 
-    for key, value in schema.items():
+    for key, value in list(schema.items()):
         if isinstance(value[0], dict):
             for item in value:
                 _remove_blank_keys(item)
@@ -291,7 +279,7 @@ def validate(data, schema, context=None):
 
     # create a copy of the context which also includes the schema keys so
     # they can be used by the validators
-    validators_context = dict(context, schema_keys=schema.keys())
+    validators_context = dict(context, schema_keys=list(schema.keys()))
 
     flattened = flatten_dict(data)
     converted_data, errors = _validate(flattened, schema, validators_context)
@@ -310,7 +298,8 @@ def validate(data, schema, context=None):
     dicts_to_process = [errors_unflattened]
     while dicts_to_process:
         dict_to_process = dicts_to_process.pop()
-        for key, value in dict_to_process.items():
+        dict_to_process_copy = copy.copy(dict_to_process)
+        for key, value in dict_to_process_copy.items():
             if not value:
                 dict_to_process.pop(key)
                 continue
@@ -398,7 +387,7 @@ def flatten_dict(data, flattened=None, old_key=None):
     flattened = flattened or {}
     old_key = old_key or []
 
-    for key, value in data.iteritems():
+    for key, value in six.iteritems(data):
         new_key = old_key + [key]
         if isinstance(value, list) and value and isinstance(value[0], dict):
             flattened = flatten_list(value, flattened, new_key)
@@ -437,7 +426,7 @@ def unflatten(data):
     '''
 
     unflattened = {}
-    convert_to_list = []
+    clean_lists = {}
 
     for flattend_key in sorted(data.keys(), key=flattened_order_key):
         current_pos = unflattened
@@ -446,8 +435,13 @@ def unflatten(data):
             try:
                 current_pos = current_pos[key]
             except IndexError:
-                new_pos = {}
-                current_pos.append(new_pos)
+                while True:
+                    new_pos = {}
+                    current_pos.append(new_pos)
+                    if key < len(current_pos):
+                        break
+                    # skipped list indexes need to be removed before returning
+                    clean_lists[id(current_pos)] = current_pos
                 current_pos = new_pos
             except KeyError:
                 new_pos = []
@@ -455,6 +449,9 @@ def unflatten(data):
                 current_pos = new_pos
         current_pos[flattend_key[-1]] = data[flattend_key]
 
+    for cl in clean_lists.values():
+        cl[:] = [i for i in cl if i]
+
     return unflattened
 
 
@@ -464,3 +461,281 @@ class MissingNullEncoder(json.JSONEncoder):
         if isinstance(obj, Missing):
             return None
         return json.JSONEncoder.default(self, obj)
+
+
+def check_dict(data_dict, select_dict, parent_path=()):
+    """
+    return list of key tuples from select_dict whose values don't match
+    corresponding values in data_dict.
+    """
+    if not isinstance(data_dict, dict):
+        return [parent_path]
+
+    unmatched = []
+    for k, v in sorted(select_dict.items()):
+        if k not in data_dict:
+            unmatched.append(parent_path + (k,))
+
+        elif isinstance(v, dict):
+            unmatched.extend(check_dict(data_dict[k], v, parent_path + (k,)))
+
+        elif isinstance(v, list):
+            unmatched.extend(check_list(data_dict[k], v, parent_path + (k,)))
+
+        elif data_dict[k] != v:
+            unmatched.append(parent_path + (k,))
+
+    return unmatched
+
+
+def check_list(data_list, select_list, parent_path=()):
+    """
+    return list of key tuples from select_list whose values don't match
+    corresponding values in data_list.
+    """
+    if not isinstance(data_list, list):
+        return [parent_path]
+
+    unmatched = []
+    for i, v in enumerate(select_list):
+        if i >= len(data_list):
+            unmatched.append(parent_path + (i,))
+
+        elif isinstance(v, dict):
+            unmatched.extend(check_dict(data_list[i], v, parent_path + (i,)))
+
+        elif isinstance(v, list):
+            unmatched.extend(check_list(data_list[i], v, parent_path + (i,)))
+
+        elif data_list[i] != v:
+            unmatched.append(parent_path + (i,))
+
+    return unmatched
+
+
+def resolve_string_key(data, string_key):
+    """
+    return (child, parent_path) if string_key is found in data
+    raise DataError on incompatible types or key not found.
+
+    supports partial-id keys for lists of dicts (minimum 5 hex digits)
+    e.g. `resources__1492a` would select the first matching resource
+    with an id field matching "1492a..."
+    """
+    parent_path = []
+    current = data
+    for k in string_key.split('__'):
+        if isinstance(current, dict):
+            if k not in current:
+                raise DataError('Unmatched key %s' % '__'.join(
+                    str(p) for p in parent_path + [k]))
+            parent_path.append(k)
+            current = current[k]
+            continue
+
+        if not isinstance(current, list):
+            raise DataError('Unmatched key %s' % '__'.join(
+                str(p) for p in parent_path + [k]))
+
+        if len(k) >= 5:
+            for i, rec in enumerate(current):
+                if not isinstance(rec, dict) or 'id' not in rec:
+                    raise DataError('Unmatched key %s' % '__'.join(
+                        str(p) for p in parent_path + [k]))
+                if rec['id'].startswith(k):
+                    parent_path.append(i)
+                    current = rec
+                    break
+            else:
+                raise DataError('Unmatched key %s' % '__'.join(
+                    str(p) for p in parent_path + [k]))
+            continue
+
+        try:
+            index = int(k)
+            if index < -len(current) or index >= len(current):
+                raise ValueError
+        except ValueError:
+            raise DataError('Unmatched key %s' % '__'.join(
+                str(p) for p in parent_path + [k]))
+
+        parent_path.append(index)
+        current = current[index]
+
+    return current, tuple(parent_path)
+
+
+def check_string_key(data_dict, string_key, value):
+    """
+    return list of key tuples from string_key whose values don't match
+    corresponding values in data_dict.
+
+    raise DataError on incompatible types such as checking for dict values
+    in a list value.
+    """
+    current, parent_path = resolve_string_key(data_dict, string_key)
+    if isinstance(value, dict):
+        return check_dict(current, value, parent_path)
+    if isinstance(value, list):
+        return check_list(current, value, parent_path)
+    if current != value:
+        return [parent_path]
+    return []
+
+
+def filter_glob_match(data_dict, glob_patterns):
+    """
+    remove keys and values from data_dict in-place based on glob patterns.
+
+    glob patterns are string_keys with optional '*' keys matching everything
+    at that level. a '+' prefix on the glob pattern indicates values to
+    protect from deletion, where the first matching pattern "wins".
+    """
+    return _filter_glob_match(data_dict, [
+        (p.startswith('+'), p.lstrip('-+').split('__'))
+        for p in glob_patterns])
+
+
+def _filter_glob_match(data, parsed_globs):
+    if isinstance(data, dict):
+        protected = {}
+        children = {}
+        for keep, globs in parsed_globs:
+            head = globs[0]
+            if head == '*':
+                if keep:
+                    protected.update(data)
+                else:
+                    data.clear()
+                continue
+            if head not in data:
+                continue
+
+            if len(globs) > 1:
+                children.setdefault(head, []).append((keep, globs[1:]))
+            elif keep:
+                protected[head] = data[head]
+            else:
+                del data[head]
+        data.update(protected)
+
+        for head in children:
+            if head not in data:
+                continue
+            _filter_glob_match(data[head], children[head])
+
+        return
+
+    elif not isinstance(data, list):
+        return
+
+    protected = set()
+    removed = set()
+    children = {}
+    for keep, globs in parsed_globs:
+        head = globs[0]
+        if head == '*':
+            if keep:
+                protected.update(set(range(len(data))) - removed)
+            else:
+                removed.update(set(range(len(data))) - protected)
+            continue
+        try:
+            child, (index,) = resolve_string_key(data, head)
+        except DataError:
+            continue
+
+        if len(globs) > 1:
+            children.setdefault(index, []).append((keep, globs[1:]))
+        elif keep:
+            if index not in removed:
+                protected.add(index)
+        else:
+            if index not in protected:
+                removed.add(index)
+
+        for head in children:
+            if head not in removed - protected:
+                _filter_glob_match(data[head], children[head])
+
+    data[:] = [e for i, e in enumerate(data) if i not in removed - protected]
+
+
+def update_merge_dict(data_dict, update_dict, parent_path=()):
+    """
+    update data_dict keys and values in-place based on update_dict.
+
+    raise DataError on incompatible types such as replacing a dict with a list
+    """
+    if not isinstance(update_dict, dict):
+        raise DataError('Expected dict for %s' % '__'.join(
+            str(p) for p in parent_path))
+
+    for k, v in update_dict.items():
+        if k not in data_dict:
+            data_dict[k] = v
+        elif isinstance(data_dict[k], dict):
+            update_merge_dict(data_dict[k], v, parent_path + (k,))
+        elif isinstance(data_dict[k], list):
+            update_merge_list(data_dict[k], v, parent_path + (k,))
+        else:
+            data_dict[k] = v
+
+
+def update_merge_list(data_list, update_list, parent_path=()):
+    """
+    update data_list entries in-place based on update_list.
+
+    raise DataError on incompatible types such as replacing a dict with a list
+    """
+    if not isinstance(update_list, list):
+        raise DataError('Expected list for %s' % '__'.join(
+            str(p) for p in parent_path))
+
+    for i, v in enumerate(update_list):
+        if i >= len(data_list):
+            data_list.append(v)
+        elif isinstance(data_list[i], dict):
+            update_merge_dict(data_list[i], v, parent_path + (i,))
+        elif isinstance(data_list[i], list):
+            update_merge_list(data_list[i], v, parent_path + (i,))
+        else:
+            data_list[i] = v
+
+
+def update_merge_string_key(data_dict, string_key, value):
+    """
+    update data_dict entries in-place based on string_key and value.
+    Also supports extending existing lists with `__extend` suffix.
+
+    raise DataError on incompatible types such as replacing a dict with a list
+    """
+
+    parts = string_key.split('__')
+    k = parts[-1]
+    string_key = '__'.join(parts[:-1])
+
+    if string_key:
+        current, parent_path = resolve_string_key(data_dict, string_key)
+    else:
+        current = data_dict
+        parent_path = ()
+
+    if isinstance(current, dict):
+        update_merge_dict(current, {k: value}, parent_path)
+    elif isinstance(current, list):
+        if k == 'extend':
+            if not isinstance(value, list):
+                raise DataError('Expected list for %s' % string_key)
+            current.extend(value)
+            return
+
+        child, (index,) = resolve_string_key(current, k)
+        if isinstance(child, dict):
+            update_merge_dict(child, value, parent_path + (index,))
+        elif isinstance(child, list):
+            update_merge_list(child, value, parent_path + (index,))
+        else:
+            current[index] = value
+    else:
+        raise DataError('Expected list or dict for %s' % string_key)
