@@ -1,10 +1,11 @@
 # encoding: utf-8
 
+import six
 from six import text_type
 
 import ckan.lib.navl.dictization_functions as df
 
-from ckan.common import _, json
+from ckan.common import _, json, config
 
 missing = df.missing
 StopOnError = df.StopOnError
@@ -17,7 +18,7 @@ def identity_converter(key, data, errors, context):
 def keep_extras(key, data, errors, context):
 
     extras = data.pop(key, {})
-    for extras_key, value in extras.iteritems():
+    for extras_key, value in six.iteritems(extras):
         data[key[:-1] + (extras_key,)] = value
 
 def not_missing(key, data, errors, context):
@@ -30,7 +31,13 @@ def not_missing(key, data, errors, context):
 def not_empty(key, data, errors, context):
 
     value = data.get(key)
-    if not value or value is missing:
+    valid_values = [False, 0, 0.0]
+
+    if value in valid_values:
+        return
+
+    if value is missing or not value:
+
         errors[key].append(_('Missing value'))
         raise StopOnError
 
@@ -64,7 +71,7 @@ def empty(key, data, errors, context):
         key_name = key[-1]
         if key_name == '__junk':
             # for junked fields, the field name is contained in the value
-            key_name = value.keys()
+            key_name = list(value.keys())
         errors[key].append(_(
             'The input field %(name)s was not expected.') % {"name": key_name})
 
@@ -85,6 +92,16 @@ def default(default_value):
 
     return callable
 
+def configured_default(config_name, default_value_if_not_configured):
+    '''When key is missing or value is an empty string or None, replace it with
+    a default value from config, or if that isn't set from the
+    default_value_if_not_configured.'''
+
+    default_value = config.get(config_name)
+    if default_value is None:
+        default_value = default_value_if_not_configured
+    return default(default_value)
+
 def ignore_missing(key, data, errors, context):
     '''If the key is missing from the data, ignore the rest of the key's
     schema.
@@ -152,7 +169,7 @@ def unicode_safe(value):
         # bytes only arrive when core ckan or plugins call
         # actions from Python code
         try:
-            return value.decode(u'utf8')
+            return six.ensure_text(value)
         except UnicodeDecodeError:
             return value.decode(u'cp1252')
     try:
@@ -163,3 +180,18 @@ def unicode_safe(value):
             return text_type(value)
         except Exception:
             return u'\N{REPLACEMENT CHARACTER}'
+
+def limit_to_configured_maximum(config_option, default_limit):
+    '''
+    If the value is over a limit, it changes it to the limit. The limit is
+    defined by a configuration option, or if that is not set, a given int
+    default_limit.
+    '''
+    def callable(key, data, errors, context):
+
+        value = convert_int(data.get(key), context)
+        limit = int(config.get(config_option, default_limit))
+        if value > limit:
+            data[key] = limit
+
+    return callable
