@@ -4,10 +4,10 @@ import logging
 import os
 import sys
 
-from ckan.common import c
-from ckan.lib import base
+from flask import Blueprint
+
+from ckan.common import c, g
 from ckan import logic
-import logic.schema
 from ckan import plugins
 import ckan.authz
 import ckan.plugins.toolkit as toolkit
@@ -26,6 +26,8 @@ _default_group_plugin = None
 _default_organization_plugin = None
 # Mapping from group-type strings to controllers
 _group_controllers = {}
+# Mapping from group-type strings to blueprints
+_group_blueprints = {}
 
 
 def reset_package_plugins():
@@ -33,6 +35,9 @@ def reset_package_plugins():
     _default_package_plugin = None
     global _package_plugins
     _package_plugins = {}
+
+
+def reset_group_plugins():
     global _default_group_plugin
     _default_group_plugin = None
     global _default_organization_plugin
@@ -49,6 +54,7 @@ def lookup_package_plugin(package_type=None):
 
     If the package type is None or cannot be found in the mapping, then the
     fallback behaviour is used.
+
     """
     if package_type is None:
         return _default_package_plugin
@@ -76,48 +82,30 @@ def lookup_group_controller(group_type=None):
     return _group_controllers.get(group_type)
 
 
-def register_package_plugins(map):
+def lookup_group_blueprints(group_type=None):
+    """
+    Returns the group blueprint
+    """
+    return _group_blueprints.get(group_type)
+
+
+def register_package_plugins():
     """
     Register the various IDatasetForm instances.
 
     This method will setup the mappings between package types and the
-    registered IDatasetForm instances. If it's called more than once an
-    exception will be raised.
+    registered IDatasetForm instances.
     """
     global _default_package_plugin
 
-    # This function should have not effect if called more than once.
-    # This should not occur in normal deployment, but it may happen when
-    # running unit tests.
-    if _default_package_plugin is not None:
-        return
-
     # Create the mappings and register the fallback behaviour if one is found.
     for plugin in plugins.PluginImplementations(plugins.IDatasetForm):
         if plugin.is_fallback():
-            if _default_package_plugin is not None:
+            if _default_package_plugin is not None and not isinstance(_default_package_plugin, DefaultDatasetForm):
                 raise ValueError("More than one fallback "
                                  "IDatasetForm has been registered")
             _default_package_plugin = plugin
-
         for package_type in plugin.package_types():
-            # Create a connection between the newly named type and the
-            # package controller
-
-            map.connect('%s_search' % package_type, '/%s' % package_type,
-                        controller='package', action='search')
-
-            map.connect('%s_new' % package_type, '/%s/new' % package_type,
-                        controller='package', action='new')
-            map.connect('%s_read' % package_type, '/%s/{id}' % package_type,
-                        controller='package', action='read')
-
-            for action in ['edit', 'authz', 'history']:
-                map.connect('%s_%s' % (package_type, action),
-                            '/%s/%s/{id}' % (package_type, action),
-                            controller='package',
-                            action=action)
-
             if package_type in _package_plugins:
                 raise ValueError("An existing IDatasetForm is "
                                  "already associated with the package type "
@@ -125,31 +113,91 @@ def register_package_plugins(map):
             _package_plugins[package_type] = plugin
 
     # Setup the fallback behaviour if one hasn't been defined.
+    set_default_package_plugin()
+
+
+def register_package_blueprints(app):
+    """
+    Register a Flask blueprint for the various IDatasetForm instances.
+
+    Actually two blueprints per IDatasetForm instance, one for the dataset routes
+    and one for the resources one.
+    """
+
+    from ckan.views.dataset import dataset, register_dataset_plugin_rules
+    from ckan.views.resource import (
+        resource, prefixed_resource,
+        register_dataset_plugin_rules as dataset_resource_rules)
+
+    registered_dataset = False
+
+    # Create the mappings and register the fallback behaviour if one is found.
+    for plugin in plugins.PluginImplementations(plugins.IDatasetForm):
+        for package_type in plugin.package_types():
+
+            if package_type == 'dataset':
+                registered_dataset = True
+
+            if package_type in app.blueprints:
+                raise ValueError(
+                    'A blueprint for has already been associated for the '
+                    'package type {}'.format(package_type))
+
+            dataset_blueprint = Blueprint(
+                package_type,
+                dataset.import_name,
+                url_prefix='/{}'.format(package_type),
+                url_defaults={'package_type': package_type})
+            if hasattr(plugin, 'prepare_dataset_blueprint'):
+                dataset_blueprint = plugin.prepare_dataset_blueprint(
+                    package_type,
+                    dataset_blueprint)
+            register_dataset_plugin_rules(dataset_blueprint)
+
+            app.register_blueprint(dataset_blueprint)
+
+            resource_blueprint = Blueprint(
+                u'{}_resource'.format(package_type),
+                resource.import_name,
+                url_prefix=u'/{}/<id>/resource'.format(package_type),
+                url_defaults={u'package_type': package_type})
+            if hasattr(plugin, 'prepare_resource_blueprint'):
+                resource_blueprint = plugin.prepare_resource_blueprint(
+                    package_type,
+                    resource_blueprint)
+            dataset_resource_rules(resource_blueprint)
+            app.register_blueprint(resource_blueprint)
+            log.debug(
+                'Registered blueprints for custom dataset type \'{}\''.format(
+                    package_type))
+
+    if not registered_dataset:
+        # core dataset blueprint not overridden
+        app.register_blueprint(dataset)
+        app.register_blueprint(resource)
+
+    # core no-package-type resource blueprint loaded last
+    app.register_blueprint(prefixed_resource)
+
+
+def set_default_package_plugin():
+    global _default_package_plugin
     if _default_package_plugin is None:
         _default_package_plugin = DefaultDatasetForm()
 
 
-def register_group_plugins(map):
+def register_group_plugins():
     """
     Register the various IGroupForm instances.
 
     This method will setup the mappings between group types and the
-    registered IGroupForm instances. If it's called more than once an
-    exception will be raised.
+    registered IGroupForm instances.
 
     It will register IGroupForm instances for both groups and organizations
     """
     global _default_group_plugin
     global _default_organization_plugin
 
-    # This function should have not effect if called more than once.
-    # This should not occur in normal deployment, but it may happen when
-    # running unit tests.
-    if (_default_group_plugin is not None or
-            _default_organization_plugin is not None):
-        return
-
-    # Create the mappings and register the fallback behaviour if one is found.
     for plugin in plugins.PluginImplementations(plugins.IGroupForm):
 
         # Get group_controller from plugin if there is one,
@@ -159,76 +207,28 @@ def register_group_plugins(map):
         except AttributeError:
             group_controller = 'group'
 
+        if hasattr(plugin, 'is_organization'):
+            is_organization = plugin.is_organization
+        else:
+            is_organization = group_controller == 'organization'
+
         if plugin.is_fallback():
-            if hasattr(plugin, 'is_organization'):
-                is_organization = plugin.is_organization
-            else:
-                is_organization = group_controller == 'organization'
 
             if is_organization:
-                if _default_organization_plugin is not None:
-                    raise ValueError("More than one fallback IGroupForm for "
-                                     "organizations has been registered")
+                if _default_organization_plugin is not None and \
+                   not isinstance(_default_organization_plugin, DefaultOrganizationForm):
+                        raise ValueError("More than one fallback IGroupForm for "
+                                         "organizations has been registered")
                 _default_organization_plugin = plugin
 
             else:
-                if _default_group_plugin is not None:
-                    raise ValueError("More than one fallback IGroupForm for "
-                                     "groups has been registered")
+                if _default_group_plugin is not None and \
+                   not isinstance(_default_group_plugin, DefaultGroupForm):
+                        raise ValueError("More than one fallback IGroupForm for "
+                                         "groups has been registered")
                 _default_group_plugin = plugin
 
         for group_type in plugin.group_types():
-            # Create the routes based on group_type here, this will
-            # allow us to have top level objects that are actually
-            # Groups, but first we need to make sure we are not
-            # clobbering an existing domain
-
-            # Our version of routes doesn't allow the environ to be
-            # passed into the match call and so we have to set it on the
-            # map instead. This looks like a threading problem waiting
-            # to happen but it is executed sequentially from inside the
-            # routing setup
-
-            map.connect('%s_index' % group_type, '/%s' % group_type,
-                        controller=group_controller, action='index')
-            map.connect('%s_new' % group_type, '/%s/new' % group_type,
-                        controller=group_controller, action='new')
-            map.connect('%s_read' % group_type, '/%s/{id}' % group_type,
-                        controller=group_controller, action='read')
-            map.connect('%s_action' % group_type,
-                        '/%s/{action}/{id}' % group_type,
-                        controller=group_controller,
-                        requirements=dict(action='|'.join(
-                            ['edit', 'authz', 'delete', 'history', 'member_new',
-                             'member_delete', 'followers', 'follow',
-                             'unfollow', 'admins', 'activity'])))
-            map.connect('%s_edit' % group_type, '/%s/edit/{id}' % group_type,
-                        controller=group_controller, action='edit',
-                        ckan_icon='pencil-square-o')
-            map.connect('%s_members' % group_type,
-                        '/%s/members/{id}' % group_type,
-                        controller=group_controller,
-                        action='members',
-                        ckan_icon='users')
-            map.connect('%s_member_new' % group_type,
-                        '/%s/member_new/{id}' % group_type,
-                        controller=group_controller,
-                        action='member_new')
-            map.connect('%s_member_delete' % group_type,
-                        '/%s/member_delete/{id}' % group_type,
-                        controller=group_controller,
-                        action='member_delete')
-            map.connect('%s_activity' % group_type,
-                        '/%s/activity/{id}/{offset}' % group_type,
-                        controller=group_controller,
-                        action='activity', ckan_icon='clock-o'),
-            map.connect('%s_about' % group_type, '/%s/about/{id}' % group_type,
-                        controller=group_controller,
-                        action='about', ckan_icon='info-circle')
-            map.connect('%s_bulk_process' % group_type,
-                        '/%s/bulk_process/{id}' % group_type,
-                        controller=group_controller,
-                        action='bulk_process', ckan_icon='sitemap')
 
             if group_type in _group_plugins:
                 raise ValueError("An existing IGroupForm is "
@@ -237,17 +237,61 @@ def register_group_plugins(map):
             _group_plugins[group_type] = plugin
             _group_controllers[group_type] = group_controller
 
-            controller_obj = None
-            # If using one of the default controllers, tell it that it is allowed
-            # to handle other group_types.
-            # Import them here to avoid circular imports.
-            if group_controller == 'group':
-                from ckan.controllers.group import GroupController as controller_obj
-            elif group_controller == 'organization':
-                from ckan.controllers.organization import OrganizationController as controller_obj
-            if controller_obj is not None:
-                controller_obj.add_group_type(group_type)
+    # Setup the fallback behaviour if one hasn't been defined.
+    set_default_group_plugin()
 
+
+def register_group_blueprints(app):
+    """
+    Register a Flask blueprint for the various IGroupForm instances.
+
+    It will register blueprints for both groups and organizations
+    """
+
+    from ckan.views.group import group, register_group_plugin_rules
+
+    for plugin in plugins.PluginImplementations(plugins.IGroupForm):
+
+        # Get group_controller from plugin if there is one,
+        # otherwise use 'group'
+        try:
+            group_controller = plugin.group_controller()
+        except AttributeError:
+            group_controller = 'group'
+
+        if hasattr(plugin, 'is_organization'):
+            is_organization = plugin.is_organization
+        else:
+            is_organization = group_controller == 'organization'
+
+        for group_type in plugin.group_types():
+
+            if group_type in (u'group', u'organization'):
+                # The default routes are registered with the core
+                # 'group' or 'organization' blueprint
+                continue
+            elif group_type in app.blueprints:
+                raise ValueError(
+                    'A blueprint for has already been associated for the '
+                    'group type {}'.format(group_type))
+
+            blueprint = Blueprint(group_type,
+                                  group.import_name,
+                                  url_prefix='/{}'.format(group_type),
+                                  url_defaults={
+                                      u'group_type': group_type,
+                                      u'is_organization': is_organization})
+            if hasattr(plugin, 'prepare_group_blueprint'):
+                blueprint = plugin.prepare_group_blueprint(
+                    group_type, blueprint)
+            register_group_plugin_rules(blueprint)
+            app.register_blueprint(blueprint)
+
+
+def set_default_group_plugin():
+    global _default_group_plugin
+    global _default_organization_plugin
+    global _group_controllers
     # Setup the fallback behaviour if one hasn't been defined.
     if _default_group_plugin is None:
         _default_group_plugin = DefaultGroupForm()
@@ -303,44 +347,24 @@ class DefaultDatasetForm(object):
 
     '''
     def create_package_schema(self):
-        return ckan.logic.schema.default_create_package_schema()
+        return logic.schema.default_create_package_schema()
 
     def update_package_schema(self):
-        return ckan.logic.schema.default_update_package_schema()
+        return logic.schema.default_update_package_schema()
 
     def show_package_schema(self):
-        return ckan.logic.schema.default_show_package_schema()
+        return logic.schema.default_show_package_schema()
 
     def setup_template_variables(self, context, data_dict):
-        authz_fn = logic.get_action('group_list_authz')
-        c.groups_authz = authz_fn(context, data_dict)
         data_dict.update({'available_only': True})
 
-        c.groups_available = authz_fn(context, data_dict)
-
-        c.licenses = [('', '')] + base.model.Package.get_license_options()
-        c.is_sysadmin = ckan.authz.is_sysadmin(c.user)
-
-        if context.get('revision_id') or context.get('revision_date'):
-            if context.get('revision_id'):
-                rev = base.model.Session.query(base.model.Revision) \
-                                .filter_by(id=context['revision_id']) \
-                                .first()
-                c.revision_date = rev.timestamp if rev else '?'
-            else:
-                c.revision_date = context.get('revision_date')
-
         ## This is messy as auths take domain object not data_dict
         context_pkg = context.get('package', None)
-        pkg = context_pkg or c.pkg
+        pkg = context_pkg or getattr(g, 'pkg', None)
+
         if pkg:
-            try:
-                if not context_pkg:
-                    context['package'] = pkg
-                logic.check_access('package_change_state', context)
-                c.auth_for_change_state = True
-            except logic.NotAuthorized:
-                c.auth_for_change_state = False
+            if not context_pkg:
+                context['package'] = pkg
 
     def new_template(self):
         return 'package/new.html'
@@ -355,7 +379,7 @@ class DefaultDatasetForm(object):
         return 'package/search.html'
 
     def history_template(self):
-        return 'package/history.html'
+        return None
 
     def resource_template(self):
         return 'package/resource_read.html'
@@ -414,13 +438,6 @@ class DefaultGroupForm(object):
         """
         return 'group/about.html'
 
-    def history_template(self):
-        """
-        Returns a string representing the location of the template to be
-        rendered for the history page
-        """
-        return 'group/history.html'
-
     def edit_template(self):
         """
         Returns a string representing the location of the template to be
@@ -524,7 +541,7 @@ class DefaultGroupForm(object):
 
         ## This is messy as auths take domain object not data_dict
         context_group = context.get('group', None)
-        group = context_group or c.group
+        group = context_group or getattr(c, 'group', '')
         if group:
             try:
                 if not context_group:
@@ -533,6 +550,9 @@ class DefaultGroupForm(object):
                 c.auth_for_change_state = True
             except logic.NotAuthorized:
                 c.auth_for_change_state = False
+        else:
+            # needs to be set to get template displayed when flask request
+            c.group = ''
 
 
 class DefaultOrganizationForm(DefaultGroupForm):
@@ -613,15 +633,24 @@ class DefaultPermissionLabels(object):
     - everyone can read public datasets "public"
     - users can read their own drafts "creator-(user id)"
     - users can read datasets belonging to their orgs "member-(org id)"
+    - users can read datasets where they are collaborators "collaborator-(dataset id)"
     '''
     def get_dataset_labels(self, dataset_obj):
         if dataset_obj.state == u'active' and not dataset_obj.private:
             return [u'public']
 
-        if dataset_obj.owner_org:
-            return [u'member-%s' % dataset_obj.owner_org]
+        if ckan.authz.check_config_permission('allow_dataset_collaborators'):
+            # Add a generic label for all this dataset collaborators
+            labels = [u'collaborator-%s' % dataset_obj.id]
+        else:
+            labels = []
 
-        return [u'creator-%s' % dataset_obj.creator_user_id]
+        if dataset_obj.owner_org:
+            labels.append(u'member-%s' % dataset_obj.owner_org)
+        else:
+            labels.append(u'creator-%s' % dataset_obj.creator_user_id)
+
+        return labels
 
     def get_user_dataset_labels(self, user_obj):
         labels = [u'public']
@@ -633,4 +662,12 @@ class DefaultPermissionLabels(object):
         orgs = logic.get_action(u'organization_list_for_user')(
             {u'user': user_obj.id}, {u'permission': u'read'})
         labels.extend(u'member-%s' % o[u'id'] for o in orgs)
+
+        if ckan.authz.check_config_permission('allow_dataset_collaborators'):
+            # Add a label for each dataset this user is a collaborator of
+            datasets = logic.get_action('package_collaborator_list_for_user')(
+                {'ignore_auth': True}, {'id': user_obj.id})
+
+            labels.extend('collaborator-%s' % d['package_id'] for d in datasets)
+
         return labels
