@@ -2,17 +2,20 @@
 
 import re
 import logging
-
-from ckan.common import config
-import pysolr
-from paste.deploy.converters import asbool
-from paste.util.multidict import MultiDict
 import six
+import pysolr
+
+from ckan.common import asbool
+from werkzeug.datastructures import MultiDict
 
-from ckan.lib.search.common import make_connection, SearchError, SearchQueryError
 import ckan.logic as logic
 import ckan.model as model
 
+from ckan.common import config
+from ckan.lib.search.common import (
+    make_connection, SearchError, SearchQueryError
+)
+
 log = logging.getLogger(__name__)
 
 _open_licenses = None
@@ -20,7 +23,7 @@ _open_licenses = None
 VALID_SOLR_PARAMETERS = set([
     'q', 'fl', 'fq', 'rows', 'sort', 'start', 'wt', 'qf', 'bf', 'boost',
     'facet', 'facet.mincount', 'facet.limit', 'facet.field',
-    'extras', 'fq_list', 'tie', 'defType', 'mm'
+    'extras', 'fq_list', 'tie', 'defType', 'mm', 'df'
 ])
 
 # for (solr) package searches, this specifies the fields that are searched
@@ -211,7 +214,7 @@ class ResourceSearchQuery(SearchQuery):
             options.update(kwargs)
 
         context = {
-            'model':model,
+            'model': model,
             'session': model.Session,
             'search_query': True,
         }
@@ -219,6 +222,7 @@ class ResourceSearchQuery(SearchQuery):
         # Transform fields into structure required by the resource_search
         # action.
         query = []
+
         for field, terms in fields.items():
             if isinstance(terms, six.string_types):
                 terms = terms.split()
@@ -264,7 +268,8 @@ class PackageSearchQuery(SearchQuery):
             'rows': 1,
             'q': 'name:"%s" OR id:"%s"' % (reference,reference),
             'wt': 'json',
-            'fq': 'site_id:"%s"' % config.get('ckan.site_id')}
+            'fq': 'site_id:"%s" ' % config.get('ckan.site_id') + '+entity_type:package'
+        }
 
         try:
             if query['q'].startswith('{!'):
@@ -312,7 +317,9 @@ class PackageSearchQuery(SearchQuery):
             query['q'] = "*:*"
 
         # number of results
-        rows_to_return = min(1000, int(query.get('rows', 10)))
+        rows_to_return = int(query.get('rows', 10))
+        # query['rows'] should be a defaulted int, due to schema, but make
+        # certain, for legacy tests
         if rows_to_return > 0:
             # #1683 Work around problem of last result being out of order
             #       in SOLR 1.4
@@ -360,6 +367,9 @@ class PackageSearchQuery(SearchQuery):
             query['mm'] = query.get('mm', '2<-1 5<80%')
             query['qf'] = query.get('qf', QUERY_FIELDS)
 
+        query.setdefault("df", "text")
+        query.setdefault("q.op", "AND")
+
         try:
             if query['q'].startswith('{!'):
                 raise SearchError('Local parameters are not supported.')
@@ -393,7 +403,7 @@ class PackageSearchQuery(SearchQuery):
         for result in self.results:
             extra_keys = filter(lambda x: x.startswith('extras_'), result.keys())
             extras = {}
-            for extra_key in extra_keys:
+            for extra_key in list(extra_keys):
                 value = result.pop(extra_key)
                 extras[extra_key[len('extras_'):]] = value
             if extra_keys:
