@@ -3,10 +3,10 @@
 import functools
 import logging
 import re
-import sys
 from collections import defaultdict
 
-import formencode.validators
+from werkzeug.local import LocalProxy
+import six
 from six import string_types, text_type
 
 import ckan.model as model
@@ -35,7 +35,10 @@ class ActionError(Exception):
         super(ActionError, self).__init__(message)
 
     def __str__(self):
-        return self.message
+        msg = self.message
+        if not isinstance(msg, six.string_types):
+            msg = str(msg)
+        return six.ensure_text(msg)
 
 
 class NotFound(ActionError):
@@ -89,13 +92,13 @@ class ValidationError(ActionError):
             ''' Do some i18n stuff on the error_dict keys '''
 
             def prettify(field_name):
-                field_name = re.sub('(?<!\w)[Uu]rl(?!\w)', 'URL',
+                field_name = re.sub(r'(?<!\w)[Uu]rl(?!\w)', 'URL',
                                     field_name.replace('_', ' ').capitalize())
                 return _(field_name.replace('_', ' '))
 
             summary = {}
 
-            for key, error in error_dict.iteritems():
+            for key, error in six.iteritems(error_dict):
                 if key == 'resources':
                     summary[_('Resources')] = _('Package resource(s) invalid')
                 elif key == 'extras':
@@ -196,7 +199,7 @@ def tuplize_dict(data_dict):
     May raise a DataError if the format of the key is incorrect.
     '''
     tuplized_dict = {}
-    for key, value in data_dict.iteritems():
+    for key, value in six.iteritems(data_dict):
         key_list = key.split('__')
         for num, key in enumerate(key_list):
             if num % 2 == 1:
@@ -211,7 +214,7 @@ def tuplize_dict(data_dict):
 def untuplize_dict(tuplized_dict):
 
     data_dict = {}
-    for key, value in tuplized_dict.iteritems():
+    for key, value in six.iteritems(tuplized_dict):
         new_key = '__'.join([str(item) for item in key])
         data_dict[new_key] = value
     return data_dict
@@ -233,6 +236,9 @@ def _prepopulate_context(context):
     except AttributeError:
         # c.user not set
         pass
+    except RuntimeError:
+        # Outside of request context
+        pass
     except TypeError:
         # c not registered
         pass
@@ -322,6 +328,33 @@ def clear_actions_cache():
 
 
 def chained_action(func):
+    '''Decorator function allowing action function to be chained.
+
+    This allows a plugin to modify the behaviour of an existing action
+    function. A Chained action function must be defined as
+    ``action_function(original_action, context, data_dict)`` where the
+    first parameter will be set to the action function in the next plugin
+    or in core ckan. The chained action may call the original_action
+    function, optionally passing different values, handling exceptions,
+    returning different values and/or raising different exceptions
+    to the caller.
+
+    Usage::
+
+        from ckan.plugins.toolkit import chained_action
+
+        @chained_action
+        @side_effect_free
+        def package_search(original_action, context, data_dict):
+            return original_action(context, data_dict)
+
+    :param func: chained action function
+    :type func: callable
+
+    :returns: chained action function
+    :rtype: callable
+
+    '''
     func.chained_action = True
     return func
 
@@ -395,7 +428,7 @@ def get_action(action):
         for part in module_path.split('.')[1:]:
             module = getattr(module, part)
         for k, v in module.__dict__.items():
-            if not k.startswith('_'):
+            if not k.startswith('_') and not isinstance(v, LocalProxy):
                 # Only load functions from the action module or already
                 # replaced functions.
                 if (hasattr(v, '__call__') and
@@ -414,9 +447,9 @@ def get_action(action):
     fetched_actions = {}
     chained_actions = defaultdict(list)
     for plugin in p.PluginImplementations(p.IActions):
-        for name, auth_function in plugin.get_actions().items():
-            if _is_chained_action(auth_function):
-                chained_actions[name].append(auth_function)
+        for name, action_function in plugin.get_actions().items():
+            if _is_chained_action(action_function):
+                chained_actions[name].append(action_function)
             elif name in resolved_action_plugins:
                 raise NameConflict(
                     'The action %r is already implemented in %r' % (
@@ -428,9 +461,9 @@ def get_action(action):
                 resolved_action_plugins[name] = plugin.name
                 # Extensions are exempted from the auth audit for now
                 # This needs to be resolved later
-                auth_function.auth_audit_exempt = True
-                fetched_actions[name] = auth_function
-    for name, func_list in chained_actions.iteritems():
+                action_function.auth_audit_exempt = True
+                fetched_actions[name] = action_function
+    for name, func_list in six.iteritems(chained_actions):
         if name not in fetched_actions and name not in _actions:
             # nothing to override from plugins or core
             raise NotFound('The action %r is not found for chained action' % (
@@ -438,7 +471,11 @@ def get_action(action):
         for func in reversed(func_list):
             # try other plugins first, fall back to core
             prev_func = fetched_actions.get(name, _actions.get(name))
-            fetched_actions[name] = functools.partial(func, prev_func)
+            new_func = functools.partial(func, prev_func)
+            # persisting attributes to the new partial function
+            for attribute, value in six.iteritems(func.__dict__):
+                setattr(new_func, attribute, value)
+            fetched_actions[name] = new_func
 
     # Use the updated ones in preference to the originals.
     _actions.update(fetched_actions)
@@ -635,6 +672,32 @@ def auth_disallow_anonymous_access(action):
 def chained_auth_function(func):
     '''
     Decorator function allowing authentication functions to be chained.
+
+    This chain starts with the last chained auth function to be registered and
+    ends with the original auth function (or a non-chained plugin override
+    version). Chained auth functions must accept an extra parameter,
+    specifically the next auth function in the chain, for example::
+
+        auth_function(next_auth, context, data_dict).
+
+    The chained auth function may call the next_auth function, optionally
+    passing different values, handling exceptions, returning different
+    values and/or raising different exceptions to the caller.
+
+    Usage::
+
+        from ckan.plugins.toolkit import chained_auth_function
+
+        @chained_auth_function
+        @auth_allow_anonymous_access
+        def user_show(next_auth, context, data_dict=None):
+            return next_auth(context, data_dict)
+
+    :param func: chained authentication function
+    :type func: callable
+
+    :returns: chained authentication function
+    :rtype: callable
     '''
     func.chained_auth_function = True
     return func
@@ -675,9 +738,9 @@ def get_validator(validator):
         _validators_cache.update(validators)
         validators = _import_module_functions('ckan.logic.validators')
         _validators_cache.update(validators)
-        _validators_cache.update({'OneOf': formencode.validators.OneOf})
         converters = _import_module_functions('ckan.logic.converters')
         _validators_cache.update(converters)
+        _validators_cache.update({'OneOf': _validators_cache['one_of']})
 
         for plugin in reversed(list(p.PluginImplementations(p.IValidators))):
             for name, fn in plugin.get_validators().items():
