@@ -6,9 +6,11 @@ import logging
 import random
 import re
 from socket import error as socket_error
-import string
+import datetime
 
-import paste.deploy.converters
+import six
+
+import ckan.common
 from sqlalchemy import func
 
 import ckan.lib.plugins as lib_plugins
@@ -21,9 +23,10 @@ import ckan.lib.dictization.model_dictize as model_dictize
 import ckan.lib.dictization.model_save as model_save
 import ckan.lib.navl.dictization_functions
 import ckan.lib.uploader as uploader
-import ckan.lib.navl.validators as validators
 import ckan.lib.mailer as mailer
 import ckan.lib.datapreview
+import ckan.lib.api_token as api_token
+import ckan.authz as authz
 
 from ckan.common import _, config
 
@@ -40,6 +43,7 @@ _check_access = logic.check_access
 _get_action = logic.get_action
 ValidationError = logic.ValidationError
 NotFound = logic.NotFound
+NotAuthorized = logic.NotAuthorized
 _get_or_bust = logic.get_or_bust
 
 
@@ -131,6 +135,7 @@ def package_create(context, data_dict):
 
     '''
     model = context['model']
+    session = context['session']
     user = context['user']
 
     if 'type' not in data_dict:
@@ -176,15 +181,9 @@ def package_create(context, data_dict):
         model.Session.rollback()
         raise ValidationError(errors)
 
-    rev = model.repo.new_revision()
-    rev.author = user
-    if 'message' in context:
-        rev.message = context['message']
-    else:
-        rev.message = _(u'REST API: Create object %s') % data.get("name")
-
     if user:
-        user_obj = model.User.by_name(user.decode('utf8'))
+
+        user_obj = model.User.by_name(six.ensure_text(user))
         if user_obj:
             data['creator_user_id'] = user_obj.id
 
@@ -200,7 +199,6 @@ def package_create(context, data_dict):
     context_org_update = context.copy()
     context_org_update['ignore_auth'] = True
     context_org_update['defer_commit'] = True
-    context_org_update['add_revision'] = False
     _get_action('package_owner_org_update')(context_org_update,
                                             {'id': pkg.id,
                                              'organization_id': pkg.owner_org})
@@ -221,21 +219,28 @@ def package_create(context, data_dict):
              'ignore_auth': True},
             {'package': data})
 
+    # Create activity
+    if not pkg.private:
+        user_obj = model.User.by_name(user)
+        if user_obj:
+            user_id = user_obj.id
+        else:
+            user_id = 'not logged in'
+
+        activity = pkg.activity_stream_item('new', user_id)
+        session.add(activity)
+
     if not context.get('defer_commit'):
         model.repo.commit()
 
-    # need to let rest api create
-    context["package"] = pkg
-    # this is added so that the rest controller can make a new location
-    context["id"] = pkg.id
-    log.debug('Created object %s' % pkg.name)
-
     return_id_only = context.get('return_id_only', False)
 
-    output = context['id'] if return_id_only \
-        else _get_action('package_show')(context, {'id': context['id']})
+    if return_id_only:
+        return pkg.id
 
-    return output
+    return _get_action('package_show')(
+        context.copy(), {'id': pkg.id}
+    )
 
 
 def resource_create(context, data_dict):
@@ -246,8 +251,6 @@ def resource_create(context, data_dict):
     :type package_id: string
     :param url: url of resource
     :type url: string
-    :param revision_id: (optional)
-    :type revision_id: string
     :param description: (optional)
     :type description: string
     :param format: (optional)
@@ -286,8 +289,9 @@ def resource_create(context, data_dict):
     if not data_dict.get('url'):
         data_dict['url'] = ''
 
+    package_show_context = dict(context, for_update=True, return_type='dict')
     pkg_dict = _get_action('package_show')(
-        dict(context, return_type='dict'),
+        package_show_context,
         {'id': package_id})
 
     _check_access('resource_create', context, data_dict)
@@ -396,7 +400,7 @@ def resource_view_create(context, data_dict):
 
     max_order = model.Session.query(
         func.max(model.ResourceView.order)
-        ).filter_by(resource_id=resource_id).first()
+    ).filter_by(resource_id=resource_id).first()
 
     order = 0
     if max_order[0] is not None:
@@ -445,7 +449,7 @@ def resource_create_default_resource_views(context, data_dict):
 
     dataset_dict = data_dict.get('package')
 
-    create_datastore_views = paste.deploy.converters.asbool(
+    create_datastore_views = ckan.common.asbool(
         data_dict.get('create_datastore_views', False))
 
     return ckan.lib.datapreview.add_views_to_resource(
@@ -482,7 +486,7 @@ def package_create_default_resource_views(context, data_dict):
 
     _check_access('package_create_default_resource_views', context, data_dict)
 
-    create_datastore_views = paste.deploy.converters.asbool(
+    create_datastore_views = ckan.common.asbool(
         data_dict.get('create_datastore_views', False))
 
     return ckan.lib.datapreview.add_views_to_dataset_resources(
@@ -542,10 +546,6 @@ def package_relationship_create(context, data_dict):
     if existing_rels:
         return _update_package_relationship(existing_rels[0],
                                             comment, context)
-    rev = model.repo.new_revision()
-    rev.author = user
-    rev.message = _(u'REST API: Create package relationship: %s %s %s') \
-        % (pkg1, rel_type, pkg2)
     rel = pkg1.add_relationship(rel_type, pkg2, comment=comment)
     if not context.get('defer_commit'):
         model.repo.commit_and_remove()
@@ -580,14 +580,6 @@ def member_create(context, data_dict=None):
     model = context['model']
     user = context['user']
 
-    rev = model.repo.new_revision()
-    rev.author = user
-    if 'message' in context:
-        rev.message = context['message']
-    else:
-        rev.message = _(u'REST API: Create member object %s') \
-            % data_dict.get('name', '')
-
     group_id, obj_id, obj_type, capacity = \
         _get_or_bust(data_dict, ['id', 'object', 'object_type', 'capacity'])
 
@@ -608,7 +600,15 @@ def member_create(context, data_dict=None):
         filter(model.Member.table_id == obj.id).\
         filter(model.Member.group_id == group.id).\
         filter(model.Member.state == 'active').first()
-    if not member:
+    if member:
+        user_obj = model.User.get(user)
+        if member.table_name == u'user' and \
+                member.table_id == user_obj.id and \
+                member.capacity == u'admin' and \
+                capacity != u'admin':
+            raise NotAuthorized("Administrators cannot revoke their "
+                                "own admin status")
+    else:
         member = model.Member(table_name=obj_type,
                               table_id=obj.id,
                               group_id=group.id,
@@ -622,6 +622,77 @@ def member_create(context, data_dict=None):
     return model_dictize.member_dictize(member, context)
 
 
+def package_collaborator_create(context, data_dict):
+    '''Make a user a collaborator in a dataset.
+
+    If the user is already a collaborator in the dataset then their
+    capacity will be updated.
+
+    Currently you must be an Admin on the dataset owner organization to
+    manage collaborators.
+
+    Note: This action requires the collaborators feature to be enabled with
+    the :ref:`ckan.auth.allow_dataset_collaborators` configuration option.
+
+    :param id: the id or name of the dataset
+    :type id: string
+    :param user_id: the id or name of the user to add or edit
+    :type user_id: string
+    :param capacity: the capacity or role of the membership. Must be one of
+        "editor" or "member". Additionally
+        if :ref:`ckan.auth.allow_admin_collaborators` is set to True, "admin"
+        is also allowed.
+    :type capacity: string
+
+    :returns: the newly created (or updated) collaborator
+    :rtype: dictionary
+    '''
+
+    model = context['model']
+
+    package_id, user_id, capacity = _get_or_bust(
+        data_dict,
+        ['id', 'user_id', 'capacity']
+    )
+
+    allowed_capacities = authz.get_collaborator_capacities()
+    if capacity not in allowed_capacities:
+        raise ValidationError(
+            _('Role must be one of "{}"').format(', '.join(
+                allowed_capacities)))
+
+    _check_access('package_collaborator_create', context, data_dict)
+
+    package = model.Package.get(package_id)
+    if not package:
+        raise NotFound(_('Dataset not found'))
+
+    user = model.User.get(user_id)
+    if not user:
+        raise NotFound(_('User not found'))
+
+    if not authz.check_config_permission('allow_dataset_collaborators'):
+        raise ValidationError(_('Dataset collaborators not enabled'))
+
+    # Check if collaborator already exists
+    collaborator = model.Session.query(model.PackageMember). \
+        filter(model.PackageMember.package_id == package.id). \
+        filter(model.PackageMember.user_id == user.id).one_or_none()
+    if not collaborator:
+        collaborator = model.PackageMember(
+            package_id=package.id,
+            user_id=user.id)
+    collaborator.capacity = capacity
+    collaborator.modified = datetime.datetime.utcnow()
+    model.Session.add(collaborator)
+    model.repo.commit()
+
+    log.info('User {} added as collaborator in package {} ({})'.format(
+        user.name, package.id, capacity))
+
+    return model_dictize.member_dictize(collaborator, context)
+
+
 def _group_or_org_create(context, data_dict, is_org=False):
     model = context['model']
     user = context['user']
@@ -632,7 +703,7 @@ def _group_or_org_create(context, data_dict, is_org=False):
     upload.update_data_dict(data_dict, 'image_url',
                             'image_upload', 'clear_upload')
     # get the schema
-    group_type = data_dict.get('type')
+    group_type = data_dict.get('type', 'organization' if is_org else 'group')
     group_plugin = lib_plugins.lookup_group_plugin(group_type)
     try:
         schema = group_plugin.form_to_db_schema_options({
@@ -659,14 +730,6 @@ def _group_or_org_create(context, data_dict, is_org=False):
         session.rollback()
         raise ValidationError(errors)
 
-    rev = model.repo.new_revision()
-    rev.author = user
-
-    if 'message' in context:
-        rev.message = context['message']
-    else:
-        rev.message = _(u'REST API: Create object %s') % data.get("name")
-
     group = model_save.group_dict_save(data, context)
 
     # Needed to let extensions know the group id
@@ -685,7 +748,7 @@ def _group_or_org_create(context, data_dict, is_org=False):
     else:
         activity_type = 'new group'
 
-    user_id = model.User.by_name(user.decode('utf8')).id
+    user_id = model.User.by_name(six.ensure_text(user)).id
 
     activity_dict = {
         'user_id': user_id,
@@ -760,7 +823,7 @@ def group_create(context, data_dict):
     :param image_url: the URL to an image to be displayed on the group's page
         (optional)
     :type image_url: string
-    :param type: the type of the group (optional),
+    :param type: the type of the group (optional, default: ``'group'``),
         :py:class:`~ckan.plugins.interfaces.IGroupForm` plugins
         associate themselves with different group types and provide custom
         group handling behaviour for these types
@@ -942,6 +1005,28 @@ def user_create(context, data_dict):
     :type fullname: string
     :param about: a description of the new user (optional)
     :type about: string
+    :param image_url: the URL to an image to be displayed on the group's page
+        (optional)
+    :type image_url: string
+    :param plugin_extras: private extra user data belonging to plugins.
+        Only sysadmin users may set this value. It should be a dict that can
+        be dumped into JSON, and plugins should namespace their extras with
+        the plugin name to avoid collisions with other plugins, eg::
+
+            {
+                "name": "test_user",
+                "email": "test@example.com",
+                "plugin_extras": {
+                    "my_plugin": {
+                        "private_extra": 1
+                    },
+                    "another_plugin": {
+                        "another_extra": True
+                    }
+                }
+            }
+    :type plugin_extras: dict
+
 
     :returns: the newly created user
     :rtype: dictionary
@@ -953,6 +1038,16 @@ def user_create(context, data_dict):
 
     _check_access('user_create', context, data_dict)
 
+    author_obj = model.User.get(context.get('user'))
+    if data_dict.get("id"):
+        is_sysadmin = (author_obj and author_obj.sysadmin)
+        if not is_sysadmin or model.User.get(data_dict["id"]):
+            data_dict.pop("id", None)
+    context.pop("user_obj", None)
+
+    upload = uploader.get_uploader('user')
+    upload.update_data_dict(data_dict, 'image_url',
+                            'image_upload', 'clear_upload')
     data, errors = _validate(data_dict, schema, context)
 
     if errors:
@@ -983,6 +1078,8 @@ def user_create(context, data_dict):
     }
     logic.get_action('activity_create')(activity_create_context, activity_dict)
 
+    upload.upload(uploader.get_max_image_size())
+
     if not context.get('defer_commit'):
         model.repo.commit()
 
@@ -994,7 +1091,14 @@ def user_create(context, data_dict):
     user_dictize_context = context.copy()
     user_dictize_context['keep_apikey'] = True
     user_dictize_context['keep_email'] = True
-    user_dict = model_dictize.user_dictize(user, user_dictize_context)
+
+    include_plugin_extras = False
+    if author_obj:
+        include_plugin_extras = author_obj.sysadmin and 'plugin_extras' in data
+    user_dict = model_dictize.user_dictize(
+        user, user_dictize_context,
+        include_plugin_extras=include_plugin_extras
+    )
 
     context['user_obj'] = user
     context['id'] = user.id
@@ -1052,7 +1156,8 @@ def user_invite(context, data_dict):
     data['name'] = name
     data['password'] = password
     data['state'] = ckan.model.State.PENDING
-    user_dict = _get_action('user_create')(context, data)
+    create_context = dict(context, ignore_auth=True)
+    user_dict = _get_action('user_create')(create_context, data)
     user = ckan.model.User.get(user_dict['id'])
     member_dict = {
         'username': user.id,
@@ -1075,9 +1180,11 @@ def user_invite(context, data_dict):
 
         _get_action('user_delete')(context, {'id': user.id})
 
-        msg = _('Error sending the invite email, ' +
-                'the user was not created: {0}').format(error)
-        raise ValidationError({'message': msg}, error_summary=msg)
+        error_dict = {
+            "message": _('Error sending the invite email, \
+                the user was not created: {0}').format(error)
+        }
+        raise ValidationError(error_dict, error_summary=error_dict)
 
     return model_dictize.user_dictize(user, context)
 
@@ -1167,18 +1274,16 @@ def activity_create(context, activity_dict, **kw):
                         'ignore_auth must be passed in the context not as '
                         'a param')
 
-    if not paste.deploy.converters.asbool(
+    if not ckan.common.asbool(
             config.get('ckan.activity_streams_enabled', 'true')):
         return
 
     model = context['model']
 
     # Any revision_id that the caller attempts to pass in the activity_dict is
-    # ignored and overwritten here.
-    if getattr(model.Session, 'revision', None):
-        activity_dict['revision_id'] = model.Session.revision.id
-    else:
-        activity_dict['revision_id'] = None
+    # ignored and removed here.
+    if 'revision_id' in activity_dict:
+        del activity_dict['revision_id']
 
     schema = context.get('schema') or \
         ckan.logic.schema.default_create_activity_schema()
@@ -1251,14 +1356,14 @@ def follow_user(context, data_dict):
 
     '''
     if 'user' not in context:
-        raise logic.NotAuthorized(_("You must be logged in to follow users"))
+        raise NotAuthorized(_("You must be logged in to follow users"))
 
     model = context['model']
     session = context['session']
 
     userobj = model.User.get(context['user'])
     if not userobj:
-        raise logic.NotAuthorized(_("You must be logged in to follow users"))
+        raise NotAuthorized(_("You must be logged in to follow users"))
 
     schema = (context.get('schema')
               or ckan.logic.schema.default_follow_user_schema())
@@ -1309,7 +1414,7 @@ def follow_dataset(context, data_dict):
     '''
 
     if 'user' not in context:
-        raise logic.NotAuthorized(
+        raise NotAuthorized(
             _("You must be logged in to follow a dataset."))
 
     model = context['model']
@@ -1317,7 +1422,7 @@ def follow_dataset(context, data_dict):
 
     userobj = model.User.get(context['user'])
     if not userobj:
-        raise logic.NotAuthorized(
+        raise NotAuthorized(
             _("You must be logged in to follow a dataset."))
 
     schema = (context.get('schema')
@@ -1450,7 +1555,7 @@ def follow_group(context, data_dict):
 
     '''
     if 'user' not in context:
-        raise logic.NotAuthorized(
+        raise NotAuthorized(
             _("You must be logged in to follow a group."))
 
     model = context['model']
@@ -1458,7 +1563,7 @@ def follow_group(context, data_dict):
 
     userobj = model.User.get(context['user'])
     if not userobj:
-        raise logic.NotAuthorized(
+        raise NotAuthorized(
             _("You must be logged in to follow a group."))
 
     schema = context.get('schema',
@@ -1489,3 +1594,55 @@ def follow_group(context, data_dict):
         follower=follower.follower_id, object=follower.object_id))
 
     return model_dictize.user_following_group_dictize(follower, context)
+
+
+def api_token_create(context, data_dict):
+    """Create new API Token for current user.
+
+    Apart from the `user` and `name` field that are required by
+    default implementation, there may be additional fields registered
+    by extensions.
+
+    :param user: name or id of the user who owns new API Token
+    :type user: string
+    :param name: distinctive name for API Token
+    :type name: string
+
+    :returns: Returns a dict with the key "token" containing the
+              encoded token value. Extensions can privide additional
+              fields via `add_extra` method of
+              :py:class:`~ckan.plugins.interfaces.IApiToken`
+    :rtype: dictionary
+
+    """
+    model = context[u'model']
+    user, name = _get_or_bust(data_dict, [u'user', u'name'])
+
+    if model.User.get(user) is None:
+        raise NotFound("User not found")
+
+    _check_access(u'api_token_create', context, data_dict)
+
+    schema = context.get(u'schema')
+    if not schema:
+        schema = api_token.get_schema()
+
+    validated_data_dict, errors = _validate(data_dict, schema, context)
+
+    if errors:
+        raise ValidationError(errors)
+
+    token_obj = model_save.api_token_save(
+        {u'user': user, u'name': name}, context
+    )
+    model.Session.commit()
+    data = {
+        u'jti': token_obj.id,
+        u'iat': api_token.into_seconds(token_obj.created_at)
+    }
+
+    data = api_token.postprocess(data, token_obj.id, validated_data_dict)
+    token = api_token.encode(data)
+
+    result = api_token.add_extra({u'token': token})
+    return result
