@@ -4,9 +4,10 @@ import ckan.logic as logic
 import ckan.authz as authz
 from ckan.common import _, config
 from ckan.logic.auth import (get_package_object, get_group_object,
-                             get_resource_object, restrict_anon)
+                             get_resource_object, get_activity_object,
+                             restrict_anon)
 from ckan.lib.plugins import get_permission_labels
-from paste.deploy.converters import asbool
+from ckan.common import asbool
 
 
 def sysadmin(context, data_dict):
@@ -40,23 +41,6 @@ def current_package_list_with_resources(context, data_dict):
     return authz.is_authorized('package_list', context, data_dict)
 
 
-def revision_list(context, data_dict):
-    # In our new model everyone can read the revison list
-    return {'success': True}
-
-
-def group_revision_list(context, data_dict):
-    return authz.is_authorized('group_show', context, data_dict)
-
-
-def organization_revision_list(context, data_dict):
-    return authz.is_authorized('group_show', context, data_dict)
-
-
-def package_revision_list(context, data_dict):
-    return authz.is_authorized('package_show', context, data_dict)
-
-
 def group_list(context, data_dict):
     # List of all active groups is visible by default
     return {'success': True}
@@ -177,11 +161,6 @@ def resource_view_list(context, data_dict):
     return authz.is_authorized('resource_show', context, data_dict)
 
 
-def revision_show(context, data_dict):
-    # No authz check in the logic function
-    return {'success': True}
-
-
 def group_show(context, data_dict):
     user = context.get('user')
     group = get_group_object(context, data_dict)
@@ -278,6 +257,94 @@ def dashboard_new_activities_count(context, data_dict):
             context, data_dict)
 
 
+def activity_list(context, data_dict):
+    '''
+    :param id: the id or name of the object (e.g. package id)
+    :type id: string
+    :param object_type: The type of the object (e.g. 'package', 'organization',
+                        'group', 'user')
+    :type object_type: string
+    :param include_data: include the data field, containing a full object dict
+        (otherwise the data field is only returned with the object's title)
+    :type include_data: boolean
+    '''
+    if data_dict['object_type'] not in ('package', 'organization', 'group',
+                                        'user'):
+        return {'success': False, 'msg': 'object_type not recognized'}
+    if (data_dict.get('include_data') and
+        not authz.check_config_permission('public_activity_stream_detail')):
+        # The 'data' field of the activity is restricted to users who are
+        # allowed to edit the object
+        show_or_update = 'update'
+    else:
+        # the activity for an object (i.e. the activity metadata) can be viewed
+        # if the user can see the object
+        show_or_update = 'show'
+    action_on_which_to_base_auth = '{}_{}'.format(
+        data_dict['object_type'], show_or_update)  # e.g. 'package_update'
+    return authz.is_authorized(action_on_which_to_base_auth, context,
+                               {'id': data_dict['id']})
+
+
+def user_activity_list(context, data_dict):
+    data_dict['object_type'] = 'user'
+    return activity_list(context, data_dict)
+
+
+def package_activity_list(context, data_dict):
+    data_dict['object_type'] = 'package'
+    return activity_list(context, data_dict)
+
+
+def group_activity_list(context, data_dict):
+    data_dict['object_type'] = 'group'
+    return activity_list(context, data_dict)
+
+
+def organization_activity_list(context, data_dict):
+    data_dict['object_type'] = 'organization'
+    return activity_list(context, data_dict)
+
+
+def activity_show(context, data_dict):
+    '''
+    :param id: the id of the activity
+    :type id: string
+    :param include_data: include the data field, containing a full object dict
+        (otherwise the data field is only returned with the object's title)
+    :type include_data: boolean
+    '''
+    activity = get_activity_object(context, data_dict)
+    # NB it would be better to have recorded an activity_type against the
+    # activity
+    if 'package' in activity.activity_type:
+        object_type = 'package'
+    else:
+        return {'success': False, 'msg': 'object_type not recognized'}
+    return activity_list(context, {
+        'id': activity.object_id,
+        'include_data': data_dict['include_data'],
+        'object_type': object_type})
+
+
+def activity_data_show(context, data_dict):
+    '''
+    :param id: the id of the activity
+    :type id: string
+    '''
+    data_dict['include_data'] = True
+    return activity_show(context, data_dict)
+
+
+def activity_diff(context, data_dict):
+    '''
+    :param id: the id of the activity
+    :type id: string
+    '''
+    data_dict['include_data'] = True
+    return activity_show(context, data_dict)
+
+
 def user_follower_list(context, data_dict):
     return authz.is_authorized('sysadmin', context, data_dict)
 
@@ -365,3 +432,149 @@ def job_list(context, data_dict):
 def job_show(context, data_dict):
     '''Show background job. Only sysadmins.'''
     return {'success': False}
+
+
+def api_token_list(context, data_dict):
+    """List all available tokens for current user.
+    """
+    user = context[u'model'].User.get(data_dict[u'user'])
+    success = user is not None and user.name == context[u'user']
+
+    return {u'success': success}
+
+
+def package_collaborator_list(context, data_dict):
+    '''Checks if a user is allowed to list the collaborators from a dataset
+
+    See :py:func:`~ckan.authz.can_manage_collaborators` for details
+    '''
+    user = context['user']
+    model = context['model']
+
+    pkg = model.Package.get(data_dict['id'])
+    user_obj = model.User.get(user)
+
+    if not authz.can_manage_collaborators(pkg.id, user_obj.id):
+        return {
+            'success': False,
+            'msg': _('User %s not authorized to list collaborators from this dataset') % user}
+
+    return {'success': True}
+
+
+def package_collaborator_list_for_user(context, data_dict):
+    '''Checks if a user is allowed to list all datasets a user is a collaborator in
+
+    The current implementation restricts to the own users themselves.
+    '''
+    user_obj = context.get('auth_user_obj')
+    if user_obj and data_dict.get('id') in (user_obj.name, user_obj.id):
+        return {'success': True}
+    return {'success': False}
+
+
+def status_show(context, data_dict):
+    '''Show information about the site's configuration. Visible to all by default.'''
+    return {'success': True}
+
+
+def dataset_followee_count(context, data_dict):
+    '''Check if the number of datasets followed by a user are visible.
+    Visible to all by default.'''
+    return {'success': True}
+
+
+def group_followee_count(context, data_dict):
+    '''Check if the number of groups followed by a user are visible.
+    Visible to all by default.'''
+    return {'success': True}
+
+
+def user_followee_count(context, data_dict):
+    '''Check if the number of users followed by a user are visible.
+    Visible to all by default.'''
+    return {'success': True}
+
+
+def followee_count(context, data_dict):
+    '''Check if the number of objects (of any type) followed by a user are visible.
+    Visible to all by default.'''
+    return {'success': True}
+
+
+def dataset_follower_count(context, data_dict):
+    '''Check if the number of followers of a dataset are visible.
+    Visible to all by default.'''
+    return {'success': True}
+
+
+def group_follower_count(context, data_dict):
+    '''Check if the number of followers of a group are visible.
+    Visible to all by default.'''
+    return {'success': True}
+
+
+def organization_follower_count(context, data_dict):
+    '''Check if the number of followers of an organization are visible.
+    Visible to all by default.'''
+    return {'success': True}
+
+
+def user_follower_count(context, data_dict):
+    '''Check if the number of followers of a user are visible.
+    Visible to all by default.'''
+    return {'success': True}
+
+
+def am_following_dataset(context, data_dict):
+    '''Check if the information about following a dataset is visible.
+    Visible to all by default.'''
+    return {'success': True}
+
+
+def am_following_group(context, data_dict):
+    '''Check if the information about following a group is visible.
+    Visible to all by default.'''
+    return {'success': True}
+
+
+def am_following_user(context, data_dict):
+    '''Check if the information about following a user is visible.
+    Visible to all by default.'''
+    return {'success': True}
+
+
+def group_package_show(context, data_dict):
+    '''Check if the set of datasets belonging to a group is visible.
+    Visible to all by default.'''
+    return {'success': True}
+
+
+def member_list(context, data_dict):
+    '''Check if the members of a given group are visible.
+    Visible to all by default.'''
+    return {'success': True}
+
+
+def resource_search(context, data_dict):
+    '''Check if resource search is allowed.
+    Allowed for all by default.'''
+    return {'success': True}
+
+
+def tag_search(context, data_dict):
+    '''Check if tag search is allowed.
+    Allowed for all by default.'''
+    return {'success': True}
+
+
+def term_translation_show(context, data_dict):
+    '''Check if the translations for the given term(s) and language(s) are visible.
+    Visible to all by default.'''
+    return {'success': True}
+
+
+def recently_changed_packages_activity_list(context, data_dict):
+    '''Check if the activity stream of all recently added or changed packages is
+    visible. Visible to all by default.'''
+    return {'success': True}
