@@ -11,9 +11,10 @@ from ckan.logic.auth.create import _check_group_auth
 
 @logic.auth_allow_anonymous_access
 def package_update(context, data_dict):
+    model = context['model']
     user = context.get('user')
-    package = logic_auth.get_package_object(context, data_dict)
 
+    package = logic_auth.get_package_object(context, data_dict)
     if package.owner_org:
         # if there is an owner org then we must have update_dataset
         # permission for that organization
@@ -34,10 +35,20 @@ def package_update(context, data_dict):
                 'create_unowned_dataset',
                 )) or authz.has_user_permission_for_some_org(
                 user, 'create_dataset')
+
     if not check1:
-        return {'success': False,
-                'msg': _('User %s not authorized to edit package %s') %
-                        (str(user), package.id)}
+        success = False
+        if authz.check_config_permission('allow_dataset_collaborators'):
+            # if org-level auth failed, check dataset-level auth
+            # (ie if user is a collaborator)
+            user_obj = model.User.get(user)
+            if user_obj:
+                success = authz.user_is_collaborator_on_dataset(
+                    user_obj.id, package.id, ['admin', 'editor'])
+        if not success:
+            return {'success': False,
+                    'msg': _('User %s not authorized to edit package %s') %
+                            (str(user), package.id)}
     else:
         check2 = _check_group_auth(context, data_dict)
         if not check2:
@@ -47,6 +58,11 @@ def package_update(context, data_dict):
 
     return {'success': True}
 
+
+def package_revise(context, data_dict):
+    return authz.is_authorized('package_update', context, data_dict['update'])
+
+
 def package_resource_reorder(context, data_dict):
     ## the action function runs package update so no need to run it twice
     return {'success': True}
@@ -78,7 +94,7 @@ def resource_view_update(context, data_dict):
     return authz.is_authorized('resource_update', context, {'id': data_dict['resource_id']})
 
 def resource_view_reorder(context, data_dict):
-    return authz.is_authorized('resource_update', context, {'id': data_dict['id']})
+    return authz.is_authorized('resource_update', context, {'id': data_dict['resource_id']})
 
 def package_relationship_update(context, data_dict):
     return authz.is_authorized('package_relationship_create',
