@@ -29,10 +29,11 @@ def validator_args(fn):
 def default_resource_schema(
         ignore_empty, unicode_safe, ignore, ignore_missing,
         remove_whitespace, if_empty_guess_format, clean_format, isodate,
-        int_validator, extras_valid_json, keep_extras):
+        int_validator, extras_valid_json, keep_extras,
+        resource_id_validator, resource_id_does_not_exist):
     return {
-        'id': [ignore_empty, unicode_safe],
-        'revision_id': [ignore_missing, unicode_safe],
+        'id': [ignore_empty, resource_id_validator,
+               resource_id_does_not_exist, unicode_safe],
         'package_id': [ignore],
         'url': [ignore_missing, unicode_safe, remove_whitespace],
         'description': [ignore_missing, unicode_safe],
@@ -60,7 +61,6 @@ def default_resource_schema(
 @validator_args
 def default_update_resource_schema(ignore):
     schema = default_resource_schema()
-    schema['revision_id'] = [ignore]
     return schema
 
 
@@ -112,7 +112,6 @@ def default_create_package_schema(
         '__before': [duplicate_extras_key, ignore],
         'id': [empty_if_not_sysadmin, ignore_missing, unicode_safe,
                package_id_does_not_exist],
-        'revision_id': [ignore],
         'name': [
             not_empty, unicode_safe, name_validator, package_name_validator],
         'title': [if_empty_same_as("name"), unicode_safe],
@@ -194,13 +193,13 @@ def default_show_package_schema(
         'position': [not_empty],
         'last_modified': [],
         'cache_last_updated': [],
-        'revision_id': [],
         'package_id': [],
         'size': [],
         'state': [],
         'mimetype': [],
         'cache_url': [],
         'name': [],
+        'description': [],
         'mimetype_inner': [],
         'resource_type': [],
         'url_type': [],
@@ -210,7 +209,6 @@ def default_show_package_schema(
         'state': [ignore_missing],
         'isopen': [ignore_missing],
         'license_url': [ignore_missing],
-        'revision_id': [],
     })
 
     schema['groups'].update({
@@ -241,7 +239,6 @@ def default_show_package_schema(
     schema['organization'] = []
     schema['owner_org'] = []
     schema['private'] = []
-    schema['revision_id'] = []
     schema['tracking_summary'] = [ignore_missing]
     schema['license_title'] = []
 
@@ -255,7 +252,6 @@ def default_group_schema(
         no_loops_in_hierarchy, ignore_not_group_admin):
     return {
         'id': [ignore_missing, unicode_safe],
-        'revision_id': [ignore],
         'name': [
             not_empty, unicode_safe, name_validator, group_name_validator],
         'title': [ignore_missing, unicode_safe],
@@ -327,7 +323,6 @@ def default_show_group_schema(
     schema['extras'] = {'__extras': [keep_extras]}
     schema['package_count'] = [ignore_missing]
     schema['packages'] = {'__extras': [keep_extras]}
-    schema['revision_id'] = []
     schema['state'] = []
     schema['users'] = {'__extras': [keep_extras]}
 
@@ -351,13 +346,13 @@ def default_extras_schema(
 
 @validator_args
 def default_relationship_schema(
-        ignore_missing, unicode_safe, not_empty, OneOf, ignore):
+        ignore_missing, unicode_safe, not_empty, one_of, ignore):
     return {
         'id': [ignore_missing, unicode_safe],
         'subject': [ignore_missing, unicode_safe],
         'object': [ignore_missing, unicode_safe],
         'type': [not_empty,
-                 OneOf(ckan.model.PackageRelationship.get_all_types())],
+                 one_of(ckan.model.PackageRelationship.get_all_types())],
         'comment': [ignore_missing, unicode_safe],
         'state': [ignore],
     }
@@ -392,9 +387,9 @@ def default_update_relationship_schema(
 @validator_args
 def default_user_schema(
         ignore_missing, unicode_safe, name_validator, user_name_validator,
-        user_password_validator, user_password_not_empty,
+        user_password_validator, user_password_not_empty, email_is_unique,
         ignore_not_sysadmin, not_empty, email_validator,
-        user_about_validator, ignore):
+        user_about_validator, ignore, boolean_validator, json_object):
     return {
         'id': [ignore_missing, unicode_safe],
         'name': [
@@ -403,14 +398,18 @@ def default_user_schema(
         'password': [user_password_validator, user_password_not_empty,
                      ignore_missing, unicode_safe],
         'password_hash': [ignore_missing, ignore_not_sysadmin, unicode_safe],
-        'email': [not_empty, unicode_safe, email_validator],
+        'email': [not_empty, email_validator, email_is_unique, unicode_safe],
         'about': [ignore_missing, user_about_validator, unicode_safe],
         'created': [ignore],
         'sysadmin': [ignore_missing, ignore_not_sysadmin],
         'apikey': [ignore],
         'reset_key': [ignore],
-        'activity_streams_email_notifications': [ignore_missing],
-        'state': [ignore_missing],
+        'activity_streams_email_notifications': [ignore_missing,
+                                                 boolean_validator],
+        'state': [ignore_missing, ignore_not_sysadmin],
+        'image_url': [ignore_missing, unicode_safe],
+        'image_display_url': [ignore_missing, unicode_safe],
+        'plugin_extras': [ignore_missing, json_object, ignore_not_sysadmin],
     }
 
 
@@ -429,8 +428,8 @@ def user_new_form_schema(
 
 @validator_args
 def user_edit_form_schema(
-        ignore_missing, unicode_safe, user_both_passwords_entered,
-        user_password_validator, user_passwords_match):
+        ignore_missing, unicode_safe, user_password_validator,
+        user_passwords_match):
     schema = default_user_schema()
 
     schema['password'] = [ignore_missing]
@@ -444,7 +443,8 @@ def user_edit_form_schema(
 @validator_args
 def default_update_user_schema(
         ignore_missing, name_validator, user_name_validator,
-        unicode_safe, user_password_validator):
+        unicode_safe, user_password_validator, email_is_unique,
+        not_empty, email_validator):
     schema = default_user_schema()
 
     schema['name'] = [
@@ -466,9 +466,9 @@ def default_generate_apikey_user_schema(
 
 @validator_args
 def default_user_invite_schema(
-        not_empty, unicode_safe):
+        not_empty, email_validator, email_is_unique):
     return {
-        'email': [not_empty, text_type],
+        'email': [not_empty, email_validator, email_is_unique, text_type],
         'group_id': [not_empty],
         'role': [not_empty],
     }
@@ -531,9 +531,6 @@ def default_create_activity_schema(
                     convert_user_name_or_id_to_id],
         'object_id': [
             not_missing, not_empty, unicode_safe, object_id_validator],
-        # We don't bother to validate revision ID, since it's always created
-        # internally by the activity_create() logic action function.
-        'revision_id': [],
         'activity_type': [not_missing, not_empty, unicode_safe,
                           activity_type_exists],
         'data': [ignore_empty, ignore_missing],
@@ -601,16 +598,30 @@ def default_pagination_schema(ignore_missing, natural_number_validator):
 
 
 @validator_args
-def default_dashboard_activity_list_schema(unicode_safe):
+def default_dashboard_activity_list_schema(
+        configured_default, natural_number_validator,
+        limit_to_configured_maximum):
     schema = default_pagination_schema()
-    schema['id'] = [unicode_safe]
+    schema['limit'] = [
+        configured_default('ckan.activity_list_limit', 31),
+        natural_number_validator,
+        limit_to_configured_maximum('ckan.activity_list_limit_max', 100)]
     return schema
 
 
 @validator_args
-def default_activity_list_schema(not_missing, unicode_safe):
+def default_activity_list_schema(
+        not_missing, unicode_safe, configured_default,
+        natural_number_validator, limit_to_configured_maximum,
+        ignore_missing, boolean_validator, ignore_not_sysadmin):
     schema = default_pagination_schema()
     schema['id'] = [not_missing, unicode_safe]
+    schema['limit'] = [
+        configured_default('ckan.activity_list_limit', 31),
+        natural_number_validator,
+        limit_to_configured_maximum('ckan.activity_list_limit_max', 100)]
+    schema['include_hidden_activity'] = [
+        ignore_missing, ignore_not_sysadmin, boolean_validator]
     return schema
 
 
@@ -619,6 +630,7 @@ def default_autocomplete_schema(
         not_missing, unicode_safe, ignore_missing, natural_number_validator):
     return {
         'q': [not_missing, unicode_safe],
+        'ignore_self': [ignore_missing],
         'limit': [ignore_missing, natural_number_validator]
     }
 
@@ -627,12 +639,13 @@ def default_autocomplete_schema(
 def default_package_search_schema(
         ignore_missing, unicode_safe, list_of_strings,
         natural_number_validator, int_validator, convert_to_json_if_string,
-        convert_to_list_if_string):
+        convert_to_list_if_string, limit_to_configured_maximum, default):
     return {
         'q': [ignore_missing, unicode_safe],
         'fl': [ignore_missing, convert_to_list_if_string],
         'fq': [ignore_missing, unicode_safe],
-        'rows': [ignore_missing, natural_number_validator],
+        'rows': [default(10), natural_number_validator,
+                 limit_to_configured_maximum('ckan.search.rows_max', 1000)],
         'sort': [ignore_missing, unicode_safe],
         'start': [ignore_missing, natural_number_validator],
         'qf': [ignore_missing, unicode_safe],
@@ -730,7 +743,7 @@ def default_update_configuration_schema(
         'ckan.site_about': [ignore_missing, unicode_safe],
         'ckan.site_intro_text': [ignore_missing, unicode_safe],
         'ckan.site_custom_css': [ignore_missing, unicode_safe],
-        'ckan.main_css': [ignore_missing, unicode_safe],
+        'ckan.theme': [ignore_missing, unicode_safe],
         'ckan.homepage_style': [ignore_missing, is_positive_integer],
         'logo_upload': [ignore_missing, unicode_safe],
         'clear_logo_upload': [ignore_missing, unicode_safe],
@@ -777,3 +790,39 @@ def job_clear_schema(ignore_missing, list_of_strings):
     return {
         u'queues': [ignore_missing, list_of_strings],
     }
+
+
+@validator_args
+def default_create_api_token_schema(
+        not_empty, unicode_safe,
+        ignore_missing, json_object, ignore_not_sysadmin):
+    return {
+        u'name': [not_empty, unicode_safe],
+        u'user': [not_empty, unicode_safe],
+        u'plugin_extras': [ignore_missing, json_object, ignore_not_sysadmin],
+    }
+
+
+@validator_args
+def package_revise_schema(
+        ignore_missing, list_of_strings,
+        collect_prefix_validate, json_or_string,
+        json_list_or_string, dict_only):
+    return {
+        u'__before': [
+            collect_prefix_validate(
+                u'match__', u'json_or_string'),
+            collect_prefix_validate(
+                u'update__', u'json_or_string')],
+        u'match': [
+            ignore_missing, json_or_string, dict_only],
+        u'filter': [
+            ignore_missing, json_list_or_string, list_of_strings],
+        u'update': [
+            ignore_missing, json_or_string, dict_only],
+        u'include': [
+            ignore_missing, json_list_or_string, list_of_strings],
+        # collect_prefix moves values to these, always dicts:
+        u'match__': [],
+        u'update__': [],
+    }
