@@ -1,16 +1,19 @@
 # encoding: utf-8
 
 import datetime
+import six
+from collections import OrderedDict
 
 import sqlalchemy as sa
 from sqlalchemy import orm
-from sqlalchemy.util import OrderedDict
+from six import string_types
+
+from ckan.model import meta, core
 
-import meta
-import core
 
 __all__ = ['DomainObject', 'DomainObjectOperation']
 
+
 class Enum(set):
     '''Simple enumeration
     e.g. Animal = Enum("dog", "cat", "horse")
@@ -26,7 +29,6 @@ class Enum(set):
 
 DomainObjectOperation = Enum('new', 'changed', 'deleted')
 
-# TODO: replace this (or at least inherit from) standard SqlalchemyMixin in vdm
 class DomainObject(object):
 
     text_search_fields = []
@@ -41,10 +43,12 @@ class DomainObject(object):
         return cls.Session.query(cls).count()
 
     @classmethod
-    def by_name(cls, name, autoflush=True):
-        obj = meta.Session.query(cls).autoflush(autoflush)\
-              .filter_by(name=name).first()
-        return obj
+    def by_name(cls, name, autoflush=True, for_update=False):
+        q = meta.Session.query(cls).autoflush(autoflush
+            ).filter_by(name=name)
+        if for_update:
+            q = q.with_for_update()
+        return q.first()
 
     @classmethod
     def text_search(cls, query, term):
@@ -79,7 +83,7 @@ class DomainObject(object):
 
     def delete(self):
         # stateful objects have this method overridden - see
-        # vmd.base.StatefulObjectMixin
+        # core.StatefulObjectMixin
         self.Session.delete(self)
 
     def purge(self):
@@ -87,6 +91,10 @@ class DomainObject(object):
         self.Session.delete(self)
 
     def as_dict(self):
+        """
+        returns: ordered dict with fields from table. Date/time values
+        are converted to strings for json compatibilty
+        """
         _dict = OrderedDict()
         table = orm.class_mapper(self.__class__).mapped_table
         for col in table.c:
@@ -98,8 +106,48 @@ class DomainObject(object):
             _dict[col.name] = val
         return _dict
 
+    def from_dict(self, _dict):
+        """
+        Loads data from dict into table.
+
+        Returns (changed, skipped) tuple. changed is a set of keys
+        that were different than the original values, i.e. changed
+        is an empty list when no values were changed by this call.
+        skipped is a dict containing any items from _dict whose keys
+        were not found in columns.
+
+        When key for a column is not present in _dict, columns marked
+        with doc='remove_if_not_provided' will have their field set
+        to N , otherwise existing field value won't be changed.
+        """
+        changed = set()
+        skipped = dict(_dict)
+        table = orm.class_mapper(self.__class__).mapped_table
+        for col in table.c:
+            if col.name.startswith('_'):
+                continue
+            if col.name in _dict:
+                value = _dict[col.name]
+                db_value = getattr(self, col.name)
+                if isinstance(db_value, datetime.datetime) and isinstance(value, string_types):
+                    db_value = db_value.isoformat()
+                if db_value != value:
+                    changed.add(col.name)
+                    setattr(self, col.name, value)
+                del skipped[col.name]
+            elif col.doc == 'remove_if_not_provided':
+                blank = None if col.nullable else ''
+                # these are expected when updating, clear when missing
+                if getattr(self, col.name) != blank:
+                    changed.add(col.name)
+                    setattr(self, col.name, blank)
+        return changed, skipped
+
+    def __lt__(self, other):
+        return self.name < other.name
+
     def __str__(self):
-        return self.__unicode__().encode('utf8')
+        return repr(self)
 
     def __unicode__(self):
         repr = u'<%s' % self.__class__.__name__
@@ -114,4 +162,4 @@ class DomainObject(object):
         return repr
 
     def __repr__(self):
-        return self.__unicode__().encode('utf-8')
+        return six.ensure_str(self.__unicode__())
