@@ -1,24 +1,20 @@
+# encoding: utf-8
+
 """
 Provides bridges between the model and plugin PluginImplementationss
 """
 import logging
+from operator import methodcaller
 
 from sqlalchemy.orm.interfaces import MapperExtension
 from sqlalchemy.orm.session import SessionExtension
 
 import ckan.plugins as plugins
 
-try:
-    from operator import methodcaller
-except ImportError:
-    def methodcaller(name, *args, **kwargs):
-        "Replaces stdlib operator.methodcaller in python <2.6"
-        def caller(obj):
-            return getattr(obj, name)(*args, **kwargs)
-        return caller
 
 log = logging.getLogger(__name__)
 
+
 class ObserverNotifier(object):
     """
     Mixin for hooking into SQLAlchemy
@@ -27,6 +23,13 @@ class ObserverNotifier(object):
 
     observers = None
 
+
+class PluginMapperExtension(MapperExtension):
+    """
+    Extension that calls plugins implementing IMapper on SQLAlchemy
+    MapperExtension events
+    """
+
     def notify_observers(self, func):
         """
         Call func(observer) for all registered observers.
@@ -34,16 +37,9 @@ class ObserverNotifier(object):
         :param func: Any callable, which will be called for each observer
         :returns: EXT_CONTINUE if no errors encountered, otherwise EXT_STOP
         """
-        for observer in self.observers:
+        for observer in plugins.PluginImplementations(plugins.IMapper):
             func(observer)
 
-class PluginMapperExtension(MapperExtension, ObserverNotifier):
-    """
-    Extension that calls plugins implementing IMapper on SQLAlchemy
-    MapperExtension events
-    """
-    observers = plugins.PluginImplementations(plugins.IMapper)
-
     def before_insert(self, mapper, connection, instance):
         return self.notify_observers(
             methodcaller('before_insert', mapper, connection, instance)
@@ -75,13 +71,21 @@ class PluginMapperExtension(MapperExtension, ObserverNotifier):
         )
 
 
-class PluginSessionExtension(SessionExtension, ObserverNotifier):
+class PluginSessionExtension(SessionExtension):
     """
-    Class that calls plugins implementing IMapper on SQLAlchemy
+    Class that calls plugins implementing ISession on SQLAlchemy
     SessionExtension events
     """
 
-    observers = plugins.PluginImplementations(plugins.ISession)
+    def notify_observers(self, func):
+        """
+        Call func(observer) for all registered observers.
+
+        :param func: Any callable, which will be called for each observer
+        :returns: EXT_CONTINUE if no errors encountered, otherwise EXT_STOP
+        """
+        for observer in plugins.PluginImplementations(plugins.ISession):
+            func(observer)
 
     def after_begin(self, session, transaction, connection):
         return self.notify_observers(
@@ -112,4 +116,3 @@ class PluginSessionExtension(SessionExtension, ObserverNotifier):
         return self.notify_observers(
             methodcaller('after_rollback', session)
         )
-
