@@ -1,69 +1,77 @@
 # encoding: utf-8
 
 import datetime
-from calendar import timegm
 import logging
+
+from sqlalchemy.sql import and_, or_
+from sqlalchemy import orm, types, Column, Table, ForeignKey
+
+from ckan.common import config
+from ckan.model import (
+    meta,
+    core,
+    license as _license,
+    types as _types,
+    domain_object,
+    activity,
+    extension,
+)
+import ckan.lib.maintain as maintain
+
+
 logger = logging.getLogger(__name__)
 
-from sqlalchemy.sql import select, and_, union, or_
-from sqlalchemy import orm
-from sqlalchemy import types, Column, Table
-from ckan.common import config
-import vdm.sqlalchemy
-
-import meta
-import core
-import license as _license
-import types as _types
-import domain_object
-import activity
-import extension
-
-import ckan.lib.maintain as maintain
-import ckan.lib.dictization as dictization
-
-__all__ = ['Package', 'package_table', 'package_revision_table',
+__all__ = ['Package', 'package_table', 'PackageMember', 'package_member_table',
            'PACKAGE_NAME_MAX_LENGTH', 'PACKAGE_NAME_MIN_LENGTH',
-           'PACKAGE_VERSION_MAX_LENGTH', 'PackageTag', 'PackageTagRevision',
-           'PackageRevision']
+           'PACKAGE_VERSION_MAX_LENGTH',
+           ]
 
 
 PACKAGE_NAME_MAX_LENGTH = 100
 PACKAGE_NAME_MIN_LENGTH = 2
 PACKAGE_VERSION_MAX_LENGTH = 100
 
-## Our Domain Object Tables
+
+# Our Domain Object Tables
 package_table = Table('package', meta.metadata,
         Column('id', types.UnicodeText, primary_key=True, default=_types.make_uuid),
         Column('name', types.Unicode(PACKAGE_NAME_MAX_LENGTH),
                nullable=False, unique=True),
-        Column('title', types.UnicodeText),
-        Column('version', types.Unicode(PACKAGE_VERSION_MAX_LENGTH)),
-        Column('url', types.UnicodeText),
-        Column('author', types.UnicodeText),
-        Column('author_email', types.UnicodeText),
-        Column('maintainer', types.UnicodeText),
-        Column('maintainer_email', types.UnicodeText),
-        Column('notes', types.UnicodeText),
-        Column('license_id', types.UnicodeText),
+        Column('title', types.UnicodeText, doc='remove_if_not_provided'),
+        Column('version', types.Unicode(PACKAGE_VERSION_MAX_LENGTH),
+               doc='remove_if_not_provided'),
+        Column('url', types.UnicodeText, doc='remove_if_not_provided'),
+        Column('author', types.UnicodeText, doc='remove_if_not_provided'),
+        Column('author_email', types.UnicodeText, doc='remove_if_not_provided'),
+        Column('maintainer', types.UnicodeText, doc='remove_if_not_provided'),
+        Column('maintainer_email', types.UnicodeText, doc='remove_if_not_provided'),
+        Column('notes', types.UnicodeText, doc='remove_if_not_provided'),
+        Column('license_id', types.UnicodeText, doc='remove_if_not_provided'),
         Column('type', types.UnicodeText, default=u'dataset'),
         Column('owner_org', types.UnicodeText),
         Column('creator_user_id', types.UnicodeText),
         Column('metadata_created', types.DateTime, default=datetime.datetime.utcnow),
         Column('metadata_modified', types.DateTime, default=datetime.datetime.utcnow),
         Column('private', types.Boolean, default=False),
+        Column('state', types.UnicodeText, default=core.State.ACTIVE),
 )
 
 
-vdm.sqlalchemy.make_table_stateful(package_table)
-package_revision_table = core.make_revisioned_table(package_table)
+package_member_table = Table(
+    'package_member',
+    meta.metadata,
+    Column('package_id', ForeignKey('package.id'), primary_key=True),
+    Column('user_id', ForeignKey('user.id'), primary_key = True),
+    Column('capacity', types.UnicodeText, nullable=False),
+    Column('modified', types.DateTime, default=datetime.datetime.utcnow),
+)
+
 
 ## -------------------
 ## Mapped classes
 
-class Package(vdm.sqlalchemy.RevisionedObjectMixin,
-        vdm.sqlalchemy.StatefulObjectMixin,
-        domain_object.DomainObject):
+class Package(core.StatefulObjectMixin,
+              domain_object.DomainObject):
 
     text_search_fields = ['name', 'title']
 
@@ -77,14 +85,17 @@ class Package(vdm.sqlalchemy.RevisionedObjectMixin,
         return meta.Session.query(cls).filter(cls.name.contains(text_query.lower()))
 
     @classmethod
-    def get(cls, reference):
+    def get(cls, reference, for_update=False):
         '''Returns a package object referenced by its id or name.'''
         if not reference:
             return None
 
-        pkg = meta.Session.query(cls).get(reference)
+        q = meta.Session.query(cls)
+        if for_update:
+            q = q.with_for_update()
+        pkg = q.get(reference)
         if pkg == None:
-            pkg = cls.by_name(reference)
+            pkg = cls.by_name(reference, for_update=for_update)
         return pkg
     # Todo: Make sure package names can't be changed to look like package IDs?
 
@@ -98,7 +109,7 @@ class Package(vdm.sqlalchemy.RevisionedObjectMixin,
         return [self]
 
     def add_resource(self, url, format=u'', description=u'', hash=u'', **kw):
-        import resource
+        from ckan.model import resource
         self.resources_all.append(resource.Resource(
             package_id=self.id,
             url=url,
@@ -135,7 +146,7 @@ class Package(vdm.sqlalchemy.RevisionedObjectMixin,
         to the vocabulary.
 
         """
-        from tag import Tag
+        from ckan.model.tag import Tag
         if not tag_name:
             return
         # Get the named tag.
@@ -228,7 +239,7 @@ class Package(vdm.sqlalchemy.RevisionedObjectMixin,
 
         Raises KeyError if the type_ is invalid.
         '''
-        import package_relationship
+        from ckan.model import package_relationship
         if type_ in package_relationship.PackageRelationship.get_forward_types():
             subject = self
             object_ = related_package
@@ -266,7 +277,7 @@ class Package(vdm.sqlalchemy.RevisionedObjectMixin,
         assert direction in ('both', 'forward', 'reverse')
         if with_package:
             assert isinstance(with_package, Package)
-        from package_relationship import PackageRelationship
+        from ckan.model.package_relationship import PackageRelationship
         forward_filters = [PackageRelationship.subject==self]
         reverse_filters = [PackageRelationship.object==self]
         if with_package:
@@ -301,7 +312,7 @@ class Package(vdm.sqlalchemy.RevisionedObjectMixin,
         non-direct relationships (such as siblings).
         @return: e.g. [(annakarenina, u"is a parent"), ...]
         '''
-        from package_relationship import PackageRelationship
+        from ckan.model.package_relationship import PackageRelationship
         rel_list = []
         for rel in self.get_relationships():
             if rel.subject == self:
@@ -368,90 +379,6 @@ class Package(vdm.sqlalchemy.RevisionedObjectMixin,
 
     license = property(get_license, set_license)
 
-    @property
-    def all_related_revisions(self):
-        '''Returns chronological list of all object revisions related to
-        this package. Includes PackageRevisions, PackageTagRevisions,
-        PackageExtraRevisions and ResourceRevisions.
-        @return List of tuples (revision, [list of object revisions of this
-                                           revision])
-                Ordered by most recent first.
-        '''
-        from tag import PackageTag
-        from resource import Resource
-        from package_extra import PackageExtra
-
-        results = {} # revision:[PackageRevision1, PackageTagRevision1, etc.]
-        for pkg_rev in self.all_revisions:
-            if not results.has_key(pkg_rev.revision):
-                results[pkg_rev.revision] = []
-            results[pkg_rev.revision].append(pkg_rev)
-        for class_ in [Resource, PackageExtra, PackageTag]:
-            rev_class = class_.__revision_class__
-            obj_revisions = meta.Session.query(rev_class).filter_by(package_id=self.id).all()
-            for obj_rev in obj_revisions:
-                if not results.has_key(obj_rev.revision):
-                    results[obj_rev.revision] = []
-                results[obj_rev.revision].append(obj_rev)
-
-        result_list = results.items()
-        return sorted(result_list, key=lambda x: x[0].timestamp, reverse=True)
-
-    @property
-    def latest_related_revision(self):
-        '''Returns the latest revision for the package and its related
-        objects.'''
-        return self.all_related_revisions[0][0]
-
-    def diff(self, to_revision=None, from_revision=None):
-        '''Overrides the diff in vdm, so that related obj revisions are
-        diffed as well as PackageRevisions'''
-        from tag import PackageTag
-        from resource import Resource
-        from package_extra import PackageExtra
-
-        results = {} # field_name:diffs
-        results.update(super(Package, self).diff(to_revision, from_revision))
-        # Iterate over PackageTag, PackageExtra, Resources etc.
-        for obj_class in [Resource, PackageExtra, PackageTag]:
-            obj_rev_class = obj_class.__revision_class__
-            # Query for object revisions related to this package
-            obj_rev_query = meta.Session.query(obj_rev_class).\
-                            filter_by(package_id=self.id).\
-                            join('revision').\
-                            order_by(core.Revision.timestamp.desc())
-            # Columns to include in the diff
-            cols_to_diff = obj_class.revisioned_fields()
-            cols_to_diff.remove('id')
-            if obj_class is Resource:
-                cols_to_diff.remove('package_id')
-            # Particular object types are better known by an invariant field
-            if obj_class is PackageTag:
-                cols_to_diff.remove('tag_id')
-            elif obj_class is PackageExtra:
-                cols_to_diff.remove('key')
-            # Iterate over each object ID
-            # e.g. for PackageTag, iterate over Tag objects
-            related_obj_ids = set([related_obj.id for related_obj in obj_rev_query.all()])
-            for related_obj_id in related_obj_ids:
-                q = obj_rev_query.filter(obj_rev_class.id==related_obj_id)
-                to_obj_rev, from_obj_rev = super(Package, self).\
-                    get_obj_revisions_to_diff(
-                    q, to_revision, from_revision)
-                for col in cols_to_diff:
-                    values = [getattr(obj_rev, col) if obj_rev else '' for obj_rev in (from_obj_rev, to_obj_rev)]
-                    value_diff = self._differ(*values)
-                    if value_diff:
-                        if obj_class.__name__ == 'PackageTag':
-                            display_id = to_obj_rev.tag.name
-                        elif obj_class.__name__ == 'PackageExtra':
-                            display_id = to_obj_rev.key
-                        else:
-                            display_id = related_obj_id[:4]
-                        key = '%s-%s-%s' % (obj_class.__name__, display_id, col)
-                        results[key] = value_diff
-        return results
-
     @property
     @maintain.deprecated('`is_private` attriute of model.Package is ' +
                          'deprecated and should not be used.  Use `private`')
@@ -505,9 +432,10 @@ class Package(vdm.sqlalchemy.RevisionedObjectMixin,
 
         return fields
 
-    def activity_stream_item(self, activity_type, revision, user_id):
+    def activity_stream_item(self, activity_type, user_id):
         import ckan.model
         import ckan.logic
+
         assert activity_type in ("new", "changed"), (
             str(activity_type))
 
@@ -527,10 +455,17 @@ class Package(vdm.sqlalchemy.RevisionedObjectMixin,
                 activity_type = 'deleted'
 
         try:
-            d = {'package': dictization.table_dictize(self,
-                context={'model': ckan.model})}
-            return activity.Activity(user_id, self.id, revision.id,
-                    "%s package" % activity_type, d)
+            # We save the entire rendered package dict so we can support
+            # viewing the past packages from the activity feed.
+            dictized_package = ckan.logic.get_action('package_show')({
+                'model': ckan.model,
+                'session': ckan.model.Session,
+                'for_view': False,  # avoid ckanext-multilingual translating it
+                'ignore_auth': True
+            }, {
+                'id': self.id,
+                'include_tracking': False
+            })
         except ckan.logic.NotFound:
             # This happens if this package is being purged and therefore has no
             # current revision.
@@ -538,20 +473,19 @@ class Package(vdm.sqlalchemy.RevisionedObjectMixin,
             # is purged.
             return None
 
-    def activity_stream_detail(self, activity_id, activity_type):
-        import ckan.model
+        actor = meta.Session.query(ckan.model.User).get(user_id)
 
-        # Handle 'deleted' objects.
-        # When the user marks a package as deleted this comes through here as
-        # a 'changed' package activity. We detect this and change it to a
-        # 'deleted' activity.
-        if activity_type == 'changed' and self.state == u'deleted':
-            activity_type = 'deleted'
-
-        package_dict = dictization.table_dictize(self,
-                context={'model':ckan.model})
-        return activity.ActivityDetail(activity_id, self.id, u"Package", activity_type,
-            {'package': package_dict })
+        return activity.Activity(
+            user_id,
+            self.id,
+            "%s package" % activity_type,
+            {
+                'package': dictized_package,
+                # We keep the acting user name around so that actions can be
+                # properly displayed even if the user is deleted in the future.
+                'actor': actor.name if actor else None
+            }
+        )
 
     def set_rating(self, user_or_ip, rating):
         '''Record a user's rating of this package.
@@ -564,8 +498,8 @@ class Package(vdm.sqlalchemy.RevisionedObjectMixin,
         @param user_or_ip - user object or an IP address string
         '''
         user = None
-        from user import User
-        from rating import Rating, MAX_RATING, MIN_RATING
+        from ckan.model.user import User
+        from ckan.model.rating import Rating, MAX_RATING, MIN_RATING
         if isinstance(user_or_ip, User):
             user = user_or_ip
             rating_query = meta.Session.query(Rating)\
@@ -598,12 +532,29 @@ class Package(vdm.sqlalchemy.RevisionedObjectMixin,
                             rating=rating)
             meta.Session.add(rating)
 
+    @property
+    @maintain.deprecated()
+    def extras_list(self):
+        '''DEPRECATED in 2.9
+
+        Returns a list of the dataset's extras, as PackageExtra object
+        NB includes deleted ones too (state='deleted')
+        '''
+        from ckan.model.package_extra import PackageExtra
+        return meta.Session.query(PackageExtra) \
+            .filter_by(package_id=self.id) \
+            .all()
+
+
+class PackageMember(domain_object.DomainObject):
+    pass
+
 
 class RatingValueException(Exception):
     pass
 
 # import here to prevent circular import
-import tag
+from ckan.model import tag
 
 meta.mapper(Package, package_table, properties={
     # delete-orphan on cascade does NOT work!
@@ -618,23 +569,16 @@ meta.mapper(Package, package_table, properties={
         ),
     },
     order_by=package_table.c.name,
-    extension=[vdm.sqlalchemy.Revisioner(package_revision_table),
-               extension.PluginMapperExtension(),
-               ],
+    extension=[extension.PluginMapperExtension()],
     )
 
-vdm.sqlalchemy.modify_base_object_mapper(Package, core.Revision, core.State)
-PackageRevision = vdm.sqlalchemy.create_object_version(meta.mapper, Package,
-        package_revision_table)
+meta.mapper(tag.PackageTag, tag.package_tag_table, properties={
+    'pkg':orm.relation(Package, backref='package_tag_all',
+        cascade='none',
+        )
+    },
+    order_by=tag.package_tag_table.c.id,
+    extension=[extension.PluginMapperExtension()],
+    )
 
-def related_packages(self):
-    return [self.continuity]
-
-PackageRevision.related_packages = related_packages
-
-
-vdm.sqlalchemy.modify_base_object_mapper(tag.PackageTag, core.Revision, core.State)
-PackageTagRevision = vdm.sqlalchemy.create_object_version(meta.mapper, tag.PackageTag,
-        tag.package_tag_revision_table)
-
-PackageTagRevision.related_packages = lambda self: [self.continuity.package]
+meta.mapper(PackageMember, package_member_table)
