@@ -3,9 +3,9 @@
 from datetime import datetime
 from sqlalchemy import types, Column, Table, UniqueConstraint
 
-import meta
-import types as _types
-import domain_object
+from ckan.model import meta
+from ckan.model import types as _types
+from ckan.model import domain_object
 
 __all__ = ['TaskStatus', 'task_status_table']
 
