@@ -78,6 +78,8 @@ class _Toolkit(object):
         'CkanCommand',
         # function for initializing CLI interfaces
         'load_config',
+        # function to promt the exception in CLI command
+        'error_shout',
         # base class for IDatasetForm plugins
         'DefaultDatasetForm',
         # base class for IGroupForm plugins
@@ -108,6 +110,10 @@ class _Toolkit(object):
         'HelperError',
         # Enqueue background job
         'enqueue_job',
+        # Email a recipient
+        'mail_recipient',
+        # Email a user
+        'mail_user',
 
         # Fully defined in this file ##
         'add_template_directory',
@@ -132,14 +138,15 @@ class _Toolkit(object):
     def _initialize(self):
         ''' get the required functions/objects, store them for later
         access and check that they match the contents dict. '''
-
+        import six
         import ckan
-        import ckan.lib.base as base
         import ckan.logic as logic
+
+        import ckan.lib.base as base
         import ckan.logic.validators as logic_validators
         import ckan.lib.navl.dictization_functions as dictization_functions
         import ckan.lib.helpers as h
-        import ckan.lib.cli as cli
+        import ckan.cli as cli
         import ckan.lib.plugins as lib_plugins
         import ckan.common as common
         from ckan.exceptions import (
@@ -147,10 +154,12 @@ class _Toolkit(object):
             HelperError
         )
         from ckan.lib.jobs import enqueue as enqueue_job
+        from ckan.lib import mailer
 
-        from paste.deploy import converters
-        import pylons
-        import webhelpers.html.tags
+        import ckan.common as converters
+        if six.PY2:
+            import ckan.lib.cli as old_cli
+            import pylons
 
         # Allow class access to these modules
         self.__class__.ckan = ckan
@@ -243,6 +252,7 @@ request body variables, cookies, the request URL, etc.
 
 '''
         t['render'] = base.render
+        t['abort'] = base.abort
         t['asbool'] = converters.asbool
         self.docstring_overrides['asbool'] = '''Convert a string (e.g. 1,
 true, True) from the config file into a boolean.
@@ -264,8 +274,7 @@ string from the config file into a list.
 For example: ``bar = toolkit.aslist(config.get('ckan.foo.bar', []))``
 
 '''
-        t['literal'] = webhelpers.html.tags.literal
-
+        t['literal'] = h.literal
         t['get_action'] = logic.get_action
         t['chained_action'] = logic.chained_action
         t['get_converter'] = logic.get_validator  # For backwards compatibility
@@ -280,23 +289,12 @@ For example: ``bar = toolkit.aslist(config.get('ckan.foo.bar', []))``
         t['StopOnError'] = dictization_functions.StopOnError
         t['UnknownValidator'] = logic.UnknownValidator
         t['Invalid'] = logic_validators.Invalid
-
-        t['CkanCommand'] = cli.CkanCommand
-        t['load_config'] = cli.load_config
         t['DefaultDatasetForm'] = lib_plugins.DefaultDatasetForm
         t['DefaultGroupForm'] = lib_plugins.DefaultGroupForm
         t['DefaultOrganizationForm'] = lib_plugins.DefaultOrganizationForm
 
-        t['response'] = pylons.response
-        self.docstring_overrides['response'] = '''The Pylons response object.
+        t['error_shout'] = cli.error_shout
 
-Pylons uses this object to generate the HTTP response it returns to the web
-browser. It has attributes like the HTTP status code, the response headers,
-content type, cookies, etc.
-
-'''
-        t['BaseController'] = base.BaseController
-        t['abort'] = base.abort
         t['redirect_to'] = h.redirect_to
         t['url_for'] = h.url_for
         t['get_or_bust'] = logic.get_or_bust
@@ -306,6 +304,8 @@ content type, cookies, etc.
         t['auth_disallow_anonymous_access'] = (
             logic.auth_disallow_anonymous_access
         )
+        t['mail_recipient'] = mailer.mail_recipient
+        t['mail_user'] = mailer.mail_user
 
         # class functions
         t['render_snippet'] = self._render_snippet
@@ -320,6 +320,21 @@ content type, cookies, etc.
         t['HelperError'] = HelperError
         t['enqueue_job'] = enqueue_job
 
+        if six.PY2:
+            t['response'] = pylons.response
+            self.docstring_overrides['response'] = '''
+The Pylons response object.
+
+Pylons uses this object to generate the HTTP response it returns to the web
+browser. It has attributes like the HTTP status code, the response headers,
+content type, cookies, etc.
+
+'''
+            t['BaseController'] = base.BaseController
+            # TODO: Sort these out
+            t['CkanCommand'] = old_cli.CkanCommand
+            t['load_config'] = old_cli.load_config
+
         # check contents list correct
         errors = set(t).symmetric_difference(set(self.contents))
         if errors:
@@ -355,8 +370,20 @@ content type, cookies, etc.
 
         The path is relative to the file calling this function.
 
+        Webassets addition: append directory to webassets load paths
+        in order to correctly rewrite relative css paths and resolve
+        public urls.
+
         '''
-        cls._add_served_directory(config, relative_path, 'extra_public_paths')
+        import ckan.lib.helpers as h
+        from ckan.lib.webassets_tools import add_public_path
+        path = cls._add_served_directory(
+            config,
+            relative_path,
+            'extra_public_paths'
+        )
+        url = h._local_url('/', locale='default')
+        add_public_path(path, url)
 
     @classmethod
     def _add_served_directory(cls, config, relative_path, config_var):
@@ -367,50 +394,67 @@ content type, cookies, etc.
         assert config_var in ('extra_template_paths', 'extra_public_paths')
         # we want the filename that of the function caller but they will
         # have used one of the available helper functions
-        frame, filename, line_number, function_name, lines, index =\
-            inspect.getouterframes(inspect.currentframe())[2]
+        # TODO: starting from python 3.5, `inspect.stack` returns list
+        # of named tuples `FrameInfo`. Don't forget to remove
+        # `getframeinfo` wrapper after migration.
+        filename = inspect.getframeinfo(inspect.stack()[2][0]).filename
 
         this_dir = os.path.dirname(filename)
         absolute_path = os.path.join(this_dir, relative_path)
-        if absolute_path not in config.get(config_var, ''):
+        if absolute_path not in config.get(config_var, '').split(','):
             if config.get(config_var):
                 config[config_var] += ',' + absolute_path
             else:
                 config[config_var] = absolute_path
+        return absolute_path
 
     @classmethod
     def _add_resource(cls, path, name):
-        '''Add a Fanstatic resource library to CKAN.
+        '''Add a WebAssets library to CKAN.
 
-        Fanstatic libraries are directories containing static resource files
-        (e.g. CSS, JavaScript or image files) that can be accessed from CKAN.
+        WebAssets libraries are directories containing static resource
+        files (e.g. CSS, JavaScript or image files) that can be
+        compiled into WebAsset Bundles.
 
         See :doc:`/theming/index` for more details.
 
         '''
         import inspect
         import os
+        from ckan.lib.webassets_tools import create_library
 
-        # we want the filename that of the function caller but they will
-        # have used one of the available helper functions
-        frame, filename, line_number, function_name, lines, index =\
-            inspect.getouterframes(inspect.currentframe())[1]
+        # we want the filename that of the function caller but they
+        # will have used one of the available helper functions
+        # TODO: starting from python 3.5, `inspect.stack` returns list
+        # of named tuples `FrameInfo`. Don't forget to remove
+        # `getframeinfo` wrapper after migration.
+        filename = inspect.getframeinfo(inspect.stack()[1][0]).filename
 
         this_dir = os.path.dirname(filename)
         absolute_path = os.path.join(this_dir, path)
-        import ckan.lib.fanstatic_resources
-        ckan.lib.fanstatic_resources.create_library(name, absolute_path)
+        create_library(name, absolute_path)
+
+        import six
+        if six.PY2:
+            # TODO: remove next two lines after dropping Fanstatic support
+            import ckan.lib.fanstatic_resources
+            ckan.lib.fanstatic_resources.create_library(name, absolute_path)
 
     @classmethod
     def _add_ckan_admin_tabs(cls, config, route_name, tab_label,
-                             config_var='ckan.admin_tabs'):
+                             config_var='ckan.admin_tabs', icon=None):
         '''
         Update 'ckan.admin_tabs' dict the passed config dict.
         '''
         # get the admin_tabs dict from the config, or an empty dict.
         admin_tabs_dict = config.get(config_var, {})
         # update the admin_tabs dict with the new values
-        admin_tabs_dict.update({route_name: tab_label})
+        admin_tabs_dict.update({
+            route_name: {
+                'label': tab_label,
+                'icon': icon
+            }
+        })
         # update the config with the updated admin_tabs dict
         config.update({config_var: admin_tabs_dict})
 
@@ -502,11 +546,9 @@ content type, cookies, etc.
                 return common.c.controller, common.c.action
             except AttributeError:
                 return (None, None)
-        # there are some routes('hello_world') that are not using blueprint
-        # For such case, let's assume that view function is a controller
-        # itself and action is None.
-        if len(endpoint) is 1:
-            return endpoint + (None,)
+        # service routes, like `static`
+        if len(endpoint) == 1:
+            return endpoint + ('index', )
         return endpoint
 
     def __getattr__(self, name):
