@@ -1,232 +1,202 @@
 {
   "": {
-    "domain": "ckan", 
-    "lang": "cs_CZ", 
+    "domain": "ckan",
+    "lang": "cs_CZ",
     "plural-forms": "nplurals=4; plural=(n == 1 && n % 1 == 0) ? 0 : (n >= 2 && n <= 4 && n % 1 == 0) ? 1: (n % 1 != 0 ) ? 2 : 3;"
-  }, 
-  "%(color)s shirt": [
-    "%(color)s shirts", 
-    "%(color)s košile", 
-    "%(color)s košile", 
-    "%(color)s košil", 
-    "%(color)s košil"
-  ], 
-  "%(num)d item": [
-    "%(num)d items", 
-    "%(num)d položka", 
-    "%(num)d položky", 
-    "%(num)d položek", 
-    "%(num)d položek"
-  ], 
-  "%(num)d missing translation": [
-    "%(num)d missing translations", 
-    "%(num)d chybějící překlad", 
-    "%(num)d chybějící překlady", 
-    "%(num)d chybějících překladů", 
-    "%(num)d chybějících překladů"
-  ], 
+  },
   "Add Filter": [
-    null, 
+    null,
     "Přidat filtr"
-  ], 
+  ],
   "An Error Occurred": [
-    null, 
+    null,
     "Nastala chyba"
-  ], 
+  ],
+  "An error occured during AJAX request. Could not load view.": [
+    null,
+    "Během požadavku AJAX došlo k chybě. Zobrazení nelze načíst."
+  ],
   "Are you sure you want to perform this action?": [
-    null, 
+    null,
     "Jste si jistí, že chcete provést tuto akci?"
-  ], 
+  ],
   "Cancel": [
-    null, 
+    null,
     "Zrušit"
-  ], 
+  ],
   "Confirm": [
-    null, 
+    null,
     "Potvrdit"
-  ], 
+  ],
   "Could not load view": [
-    null, 
+    null,
     "Nelze načíst pohled"
-  ], 
+  ],
   "DataProxy returned an error": [
-    null, 
+    null,
     "Nastala chyba v DataProxy"
-  ], 
+  ],
   "DataStore returned an error": [
-    null, 
+    null,
     "Nastala chyba v DataStore"
-  ], 
+  ],
   "Edit": [
-    null, 
+    null,
     "Upravit"
-  ], 
+  ],
   "Failed to load data API information": [
-    null, 
+    null,
     "Pokus o získání informací pomocí API selhal"
-  ], 
+  ],
   "File": [
-    null, 
+    null,
     "Soubor"
-  ], 
+  ],
   "Filters": [
-    null, 
+    null,
     "Filtry"
-  ], 
+  ],
   "Follow": [
-    null, 
+    null,
     "Sledovat"
-  ], 
+  ],
   "Graph": [
-    null, 
+    null,
     "Graf"
-  ], 
+  ],
   "Grid": [
-    null, 
+    null,
     "Mřížka"
-  ], 
+  ],
   "Hide": [
-    null, 
+    null,
     "Skrýt"
-  ], 
+  ],
   "Image": [
-    null, 
+    null,
     "Obrázek"
-  ], 
+  ],
   "Input is too short, must be at least one character": [
-    "Input is too short, must be at least %(num)d characters", 
-    "Vstup je příliš krátky, musí mít alespoň jeden znak", 
-    "Vstup je příliš krátky, musí mít alespoň %(num)d znaky", 
-    "Vstup je příliš krátky, musí mít alespoň %(num)d znaků", 
+    "Input is too short, must be at least %(num)d characters",
+    "Vstup je příliš krátky, musí mít alespoň jeden znak",
+    "Vstup je příliš krátky, musí mít alespoň %(num)d znaky",
+    "Vstup je příliš krátky, musí mít alespoň %(num)d znaků",
     "Vstup je příliš krátky, musí mít alespoň %(num)d znaků"
-  ], 
+  ],
   "Link": [
-    null, 
+    null,
     "Odkaz"
-  ], 
+  ],
   "Link to a URL on the internet (you can also link to an API)": [
-    null, 
+    null,
     "Odkaz na internetovou URL adresu (můžete také zadat odkaz na API)"
-  ], 
+  ],
   "Loading...": [
-    null, 
+    null,
     "Nahrávám..."
-  ], 
+  ],
   "Map": [
-    null, 
+    null,
     "Mapa"
-  ], 
+  ],
   "No matches found": [
-    null, 
+    null,
     "Nenalezena žádná shoda"
-  ], 
+  ],
   "Please Confirm Action": [
-    null, 
+    null,
     "Prosím potvrďte akci"
-  ], 
+  ],
+  "Please select the file to upload again": [
+    null,
+    "Vyberte soubor, který chcete znovu nahrát"
+  ],
   "Remove": [
-    null, 
+    null,
     "Odstranit"
-  ], 
+  ],
+  "Reorder resource view": [
+    null,
+    "Změnit zobrazení prostředků"
+  ],
+  "Reorder resources": [
+    null,
+    "Změnit pořadí zdrojů"
+  ],
   "Resource uploaded": [
-    null, 
+    null,
     "Datový zdroj nahrán"
-  ], 
+  ],
   "Save order": [
-    null, 
+    null,
     "Uložit pořadí"
-  ], 
+  ],
   "Saving...": [
-    null, 
+    null,
     "Ukládám..."
-  ], 
+  ],
   "Select a field": [
-    null, 
+    null,
     "Vyberte pole"
-  ], 
+  ],
   "Show more": [
-    null, 
+    null,
     "Ukázat více"
-  ], 
+  ],
   "Start typing…": [
-    null, 
+    null,
     "Začněte psát..."
-  ], 
+  ],
   "There are unsaved modifications to this form": [
-    null, 
+    null,
     "Tento formulář obsahuje neuložené změny"
-  ], 
+  ],
   "There is no API data to load for this resource": [
-    null, 
+    null,
     "Tento datový zdroj neobsahuje žádná data, která lze poskytnou přes API"
-  ], 
+  ],
   "URL": [
-    null, 
+    null,
     "URL"
-  ], 
+  ],
   "Unable to authenticate upload": [
-    null, 
+    null,
     "Nastala chyba autentizace při nahrávání dat"
-  ], 
+  ],
   "Unable to get data for uploaded file": [
-    null, 
+    null,
     "Nelze získat data z nahraného souboru"
-  ], 
+  ],
   "Unable to upload file": [
-    null, 
+    null,
     "Nelze nahrát soubor"
-  ], 
+  ],
   "Unfollow": [
-    null, 
+    null,
     "Přestat sledovat"
-  ], 
+  ],
   "Upload": [
-    null, 
+    null,
     "Nahrát"
-  ], 
+  ],
   "Upload a file": [
-    null, 
+    null,
     "Nahrát soubor"
-  ], 
+  ],
   "Upload a file on your computer": [
-    null, 
+    null,
     "Nahrát soubor z počítače"
-  ], 
+  ],
   "You are uploading a file. Are you sure you want to navigate away and stop this upload?": [
-    null, 
+    null,
     "Právě nahráváte soubor. Jste si opravdu jistí, že chcete tuto stránku opustit a ukončit tak nahrávání?"
-  ], 
-  "bar": [
-    "bars", 
-    "něco", 
-    "něco", 
-    "něco", 
-    "něco"
-  ], 
+  ],
+  "You can rearrange the resources by dragging them using the arrow icon. Drag the resource to the right and place it to the desired location on the list. When you are done, click the \"Save order\" -button.": [
+    null,
+    "Uspořádání zdrojů můžete změnit jejich přetažením pomocí ikony šipky. Přetáhněte zdroj doprava a umístěte jej na požadované místo v seznamu. Po dokončení klikněte na tlačítko „Uložit pořadí“."
+  ],
   "error loading view": [
-    null, 
+    null,
     "Chyba při načítání pohledu"
-  ], 
-  "foo": [
-    null, 
-    "něco"
-  ], 
-  "hello %(name)s!": [
-    null, 
-    "Vítejte %(name)s!"
-  ], 
-  "no %(attr)s translation": [
-    "no %(attr)s translations", 
-    "chybí překlad %(attr)s", 
-    "chybí překlady %(attr)s", 
-    "chybí překlady %(attr)s", 
-    "chybí překlady %(attr)s"
-  ], 
-  "no translation": [
-    "no translations", 
-    "žádný překlad", 
-    "žádné překlady", 
-    "žádné překlady", 
-    "žádné překlady"
   ]
 }
\ No newline at end of file
