@@ -1,210 +1,180 @@
 {
   "": {
-    "domain": "ckan", 
-    "lang": "en_GB", 
+    "domain": "ckan",
+    "lang": "en_GB",
     "plural-forms": "nplurals=2; plural=(n != 1);"
-  }, 
-  "%(color)s shirt": [
-    "%(color)s shirts", 
-    "", 
-    ""
-  ], 
-  "%(num)d item": [
-    "%(num)d items", 
-    "", 
-    ""
-  ], 
-  "%(num)d missing translation": [
-    "%(num)d missing translations", 
-    "", 
-    ""
-  ], 
+  },
   "Add Filter": [
-    null, 
+    null,
     "Add Filter"
-  ], 
+  ],
   "An Error Occurred": [
-    null, 
+    null,
     "An Error Occurred"
-  ], 
+  ],
   "Are you sure you want to perform this action?": [
-    null, 
+    null,
     "Are you sure you want to perform this action?"
-  ], 
+  ],
   "Cancel": [
-    null, 
+    null,
     "Cancel"
-  ], 
+  ],
   "Confirm": [
-    null, 
+    null,
     "Confirm"
-  ], 
+  ],
   "Could not load view": [
-    null, 
+    null,
     "Could not load view"
-  ], 
+  ],
   "DataProxy returned an error": [
-    null, 
+    null,
     "DataProxy returned an error"
-  ], 
+  ],
   "DataStore returned an error": [
-    null, 
+    null,
     "DataStore returned an error"
-  ], 
+  ],
   "Edit": [
-    null, 
+    null,
     "Edit"
-  ], 
+  ],
   "Failed to load data API information": [
-    null, 
+    null,
     "Failed to load data API information"
-  ], 
+  ],
   "File": [
-    null, 
+    null,
     "File"
-  ], 
+  ],
   "Filters": [
-    null, 
+    null,
     "Filters"
-  ], 
+  ],
   "Follow": [
-    null, 
+    null,
     "Follow"
-  ], 
+  ],
   "Graph": [
-    null, 
+    null,
     "Graph"
-  ], 
+  ],
   "Grid": [
-    null, 
+    null,
     "Grid"
-  ], 
+  ],
   "Hide": [
-    null, 
+    null,
     "Hide"
-  ], 
+  ],
   "Image": [
-    null, 
+    null,
     "Image"
-  ], 
+  ],
   "Input is too short, must be at least one character": [
-    "Input is too short, must be at least %(num)d characters", 
-    "Input is too short, must be at least one character", 
+    "Input is too short, must be at least %(num)d characters",
+    "Input is too short, must be at least one character",
     "Input is too short, must be at least %(num)d characters"
-  ], 
+  ],
   "Link": [
-    null, 
+    null,
     "Link"
-  ], 
+  ],
   "Link to a URL on the internet (you can also link to an API)": [
-    null, 
+    null,
     "Link to a URL on the internet (you can also link to an API)"
-  ], 
+  ],
   "Loading...": [
-    null, 
+    null,
     "Loading..."
-  ], 
+  ],
   "Map": [
-    null, 
+    null,
     "Map"
-  ], 
+  ],
   "No matches found": [
-    null, 
+    null,
     "No matches found"
-  ], 
+  ],
   "Please Confirm Action": [
-    null, 
+    null,
     "Please Confirm Action"
-  ], 
+  ],
   "Remove": [
-    null, 
+    null,
     "Remove"
-  ], 
+  ],
   "Resource uploaded": [
-    null, 
+    null,
     "Resource uploaded"
-  ], 
+  ],
   "Save order": [
-    null, 
+    null,
     "Save order"
-  ], 
+  ],
   "Saving...": [
-    null, 
+    null,
     "Saving..."
-  ], 
+  ],
   "Select a field": [
-    null, 
+    null,
     "Select a field"
-  ], 
+  ],
   "Show more": [
-    null, 
+    null,
     "Show more"
-  ], 
+  ],
   "Start typing…": [
-    null, 
+    null,
     "Start typing…"
-  ], 
+  ],
   "There are unsaved modifications to this form": [
-    null, 
+    null,
     "There are unsaved modifications to this form"
-  ], 
+  ],
   "There is no API data to load for this resource": [
-    null, 
+    null,
     "There is no API data to load for this resource"
-  ], 
+  ],
   "URL": [
-    null, 
+    null,
     "URL"
-  ], 
+  ],
   "Unable to authenticate upload": [
-    null, 
+    null,
     "Unable to authenticate upload"
-  ], 
+  ],
   "Unable to get data for uploaded file": [
-    null, 
+    null,
     "Unable to get data for uploaded file"
-  ], 
+  ],
   "Unable to upload file": [
-    null, 
+    null,
     "Unable to upload file"
-  ], 
+  ],
   "Unfollow": [
-    null, 
+    null,
     "Unfollow"
-  ], 
+  ],
   "Upload": [
-    null, 
+    null,
     "Upload"
-  ], 
+  ],
   "Upload a file": [
-    null, 
+    null,
     "Upload a file"
-  ], 
+  ],
   "Upload a file on your computer": [
-    null, 
+    null,
     "Upload a file on your computer"
-  ], 
+  ],
   "You are uploading a file. Are you sure you want to navigate away and stop this upload?": [
-    null, 
+    null,
     "You are uploading a file. Are you sure you want to navigate away and stop this upload?"
-  ], 
-  "bar": [
-    "bars", 
-    "", 
-    ""
-  ], 
+  ],
   "error loading view": [
-    null, 
+    null,
     "error loading view"
-  ], 
-  "no %(attr)s translation": [
-    "no %(attr)s translations", 
-    "", 
-    ""
-  ], 
-  "no translation": [
-    "no translations", 
-    "", 
-    ""
   ]
 }
\ No newline at end of file
