@@ -1,218 +1,201 @@
 {
   "": {
-    "domain": "ckan", 
-    "lang": "es", 
-    "plural-forms": "nplurals=2; plural=(n != 1);"
-  }, 
-  "%(color)s shirt": [
-    "%(color)s shirts", 
-    "%(color)s shirt", 
-    "%(color)s shirts"
-  ], 
-  "%(num)d item": [
-    "%(num)d items", 
-    "%(num)d item", 
-    "%(num)d items"
-  ], 
-  "%(num)d missing translation": [
-    "%(num)d missing translations", 
-    "Falta %(num)d traducción", 
-    "Faltan %(num)d traducciones"
-  ], 
+    "domain": "ckan",
+    "lang": "es",
+    "plural-forms": "nplurals=3; plural=n == 1 ? 0 : n != 0 && n % 1000000 == 0 ? 1 : 2;"
+  },
   "Add Filter": [
-    null, 
+    null,
     "Agregar Filtro"
-  ], 
+  ],
   "An Error Occurred": [
-    null, 
+    null,
     "Ocurrió un error"
-  ], 
+  ],
+  "An error occured during AJAX request. Could not load view.": [
+    null,
+    "Ocurrió un error en la petición AJAX. No se puede cargar la vista."
+  ],
   "Are you sure you want to perform this action?": [
-    null, 
+    null,
     "¿Está seguro de que desea realizar esta acción?"
-  ], 
+  ],
   "Cancel": [
-    null, 
+    null,
     "Cancelar"
-  ], 
+  ],
   "Confirm": [
-    null, 
+    null,
     "Confirmar"
-  ], 
+  ],
   "Could not load view": [
-    null, 
+    null,
     "No se pudo cargar la vista"
-  ], 
+  ],
   "DataProxy returned an error": [
-    null, 
+    null,
     "DataProxy ha retornado un error"
-  ], 
+  ],
   "DataStore returned an error": [
-    null, 
+    null,
     "DataStore ha retornado un error"
-  ], 
+  ],
   "Edit": [
-    null, 
+    null,
     "Editar"
-  ], 
+  ],
   "Failed to load data API information": [
-    null, 
+    null,
     "Falló carga de información de datos de API"
-  ], 
+  ],
   "File": [
-    null, 
+    null,
     "Fichero"
-  ], 
+  ],
   "Filters": [
-    null, 
+    null,
     "Filtros"
-  ], 
+  ],
   "Follow": [
-    null, 
+    null,
     "Seguir"
-  ], 
+  ],
   "Graph": [
-    null, 
+    null,
     "Gráfico"
-  ], 
+  ],
   "Grid": [
-    null, 
+    null,
     "Tabla"
-  ], 
+  ],
   "Hide": [
-    null, 
+    null,
     "Esconder"
-  ], 
+  ],
   "Image": [
-    null, 
+    null,
     "Imagen"
-  ], 
+  ],
   "Input is too short, must be at least one character": [
-    "Input is too short, must be at least %(num)d characters", 
-    "Ingreso es demasiado corto, debe tener al menos un carácter", 
+    "Input is too short, must be at least %(num)d characters",
+    "Ingreso es demasiado corto, debe tener al menos un carácter",
+    "Ingreso es demasiado corto, debe tener al menos %(num)d carácteres",
     "Ingreso es demasiado corto, debe tener al menos %(num)d carácteres"
-  ], 
+  ],
   "Link": [
-    null, 
+    null,
     "Enlace"
-  ], 
+  ],
   "Link to a URL on the internet (you can also link to an API)": [
-    null, 
+    null,
     "Enlazar a una URL de internet (también puede enlazar a una API)"
-  ], 
+  ],
   "Loading...": [
-    null, 
+    null,
     "Cargando ..."
-  ], 
+  ],
   "Map": [
-    null, 
+    null,
     "Mapa"
-  ], 
+  ],
   "No matches found": [
-    null, 
+    null,
     "No se encontraron coincidencias"
-  ], 
+  ],
   "Please Confirm Action": [
-    null, 
+    null,
     "Por favor confirme la acción"
-  ], 
+  ],
+  "Please select the file to upload again": [
+    null,
+    "Por favor seleccione de nuevo el fichero para cargar"
+  ],
   "Remove": [
-    null, 
+    null,
     "Quitar"
-  ], 
+  ],
+  "Reorder resource view": [
+    null,
+    "Reordenar vista de recurso"
+  ],
+  "Reorder resources": [
+    null,
+    "Reordenar recursos"
+  ],
   "Resource uploaded": [
-    null, 
+    null,
     "Recurso subido"
-  ], 
+  ],
   "Save order": [
-    null, 
+    null,
     "Guardar orden"
-  ], 
+  ],
   "Saving...": [
-    null, 
+    null,
     "Guardando..."
-  ], 
+  ],
   "Select a field": [
-    null, 
+    null,
     "Seleccione un campo"
-  ], 
+  ],
   "Show more": [
-    null, 
+    null,
     "Mostrar más"
-  ], 
+  ],
   "Start typing…": [
-    null, 
+    null,
     "Comience a escribir..."
-  ], 
+  ],
   "There are unsaved modifications to this form": [
-    null, 
+    null,
     "Existen modificaciones no guardadas para este formulario"
-  ], 
+  ],
   "There is no API data to load for this resource": [
-    null, 
+    null,
     "No existen datos de API para cargar para este recurso"
-  ], 
+  ],
   "URL": [
-    null, 
+    null,
     "URL"
-  ], 
+  ],
   "Unable to authenticate upload": [
-    null, 
+    null,
     "No se pudo autenticar subida"
-  ], 
+  ],
   "Unable to get data for uploaded file": [
-    null, 
+    null,
     "No se pudo obtener datos para el archivo subido"
-  ], 
+  ],
   "Unable to upload file": [
-    null, 
+    null,
     "No se pudo subir el archivo"
-  ], 
+  ],
   "Unfollow": [
-    null, 
+    null,
     "Dejar de seguir"
-  ], 
+  ],
   "Upload": [
-    null, 
+    null,
     "Subir"
-  ], 
+  ],
   "Upload a file": [
-    null, 
+    null,
     "Sube un archivo"
-  ], 
+  ],
   "Upload a file on your computer": [
-    null, 
+    null,
     "Subir un fichero a su PC"
-  ], 
+  ],
   "You are uploading a file. Are you sure you want to navigate away and stop this upload?": [
-    null, 
+    null,
     "Estás subiendo un fichero. Estás seguro que quieres salir y parar esta subida?"
-  ], 
-  "bar": [
-    "bars", 
-    "barra", 
-    "barras"
-  ], 
+  ],
+  "You can rearrange the resources by dragging them using the arrow icon. Drag the resource to the right and place it to the desired location on the list. When you are done, click the \"Save order\" -button.": [
+    null,
+    "Puedes reorganizar los recursos arrastrándolos, utilizando para ello el icono de la flecha. Arrastre el recurso a la derecha y suéltelo en la posición deseada de la lista. Cuando hayas terminado, haz click en el botón \"Guardar orden\"."
+  ],
   "error loading view": [
-    null, 
+    null,
     "error cargando la vista"
-  ], 
-  "foo": [
-    null, 
-    "foo"
-  ], 
-  "hello %(name)s!": [
-    null, 
-    "Hola %(name)s!"
-  ], 
-  "no %(attr)s translation": [
-    "no %(attr)s translations", 
-    "no %(attr)s translation", 
-    "no %(attr)s translations"
-  ], 
-  "no translation": [
-    "no translations", 
-    "sin traducción", 
-    "sin traducciones"
   ]
 }
\ No newline at end of file
