@@ -1,208 +1,182 @@
 {
   "": {
-    "domain": "ckan", 
-    "lang": "ru", 
+    "domain": "ckan",
+    "lang": "ru",
     "plural-forms": "nplurals=4; plural=(n%10==1 && n%100!=11 ? 0 : n%10>=2 && n%10<=4 && (n%100<12 || n%100>14) ? 1 : n%10==0 || (n%10>=5 && n%10<=9) || (n%100>=11 && n%100<=14)? 2 : 3);"
-  }, 
-  "%(color)s shirt": [
-    "%(color)s shirts", 
-    "", 
-    "", 
-    "", 
-    ""
-  ], 
-  "%(num)d item": [
-    "%(num)d items", 
-    "", 
-    "", 
-    "", 
-    ""
-  ], 
-  "%(num)d missing translation": [
-    "%(num)d missing translations", 
-    "", 
-    "", 
-    "", 
-    ""
-  ], 
+  },
   "Add Filter": [
-    null, 
+    null,
     "Добавить фильтр"
-  ], 
+  ],
   "An Error Occurred": [
-    null, 
+    null,
     "Возникла ошибка"
-  ], 
+  ],
   "Are you sure you want to perform this action?": [
-    null, 
+    null,
     "Вы уверены, что хотите выполнить это действие?"
-  ], 
+  ],
   "Cancel": [
-    null, 
+    null,
     "Отменить"
-  ], 
+  ],
   "Confirm": [
-    null, 
+    null,
     "Потверждение"
-  ], 
+  ],
   "Could not load view": [
-    null, 
+    null,
     "Невозможно загрузить просмотр"
-  ], 
+  ],
+  "DataProxy returned an error": [
+    null,
+    "DataProxy не сумел обработать запрос"
+  ],
+  "DataStore returned an error": [
+    null,
+    "DataStore не сумел обработать запрос"
+  ],
   "Edit": [
-    null, 
+    null,
     "Редактировать"
-  ], 
+  ],
   "Failed to load data API information": [
-    null, 
+    null,
     "Не удалось загрузить данные API "
-  ], 
+  ],
   "File": [
-    null, 
+    null,
     "Файл"
-  ], 
+  ],
   "Filters": [
-    null, 
+    null,
     "Фильтры"
-  ], 
+  ],
   "Follow": [
-    null, 
+    null,
     "Следить"
-  ], 
+  ],
   "Graph": [
-    null, 
+    null,
     "График"
-  ], 
+  ],
   "Grid": [
-    null, 
+    null,
     "Сетка"
-  ], 
+  ],
   "Hide": [
-    null, 
+    null,
     "Спрятать"
-  ], 
+  ],
   "Image": [
-    null, 
+    null,
     "Картинка"
-  ], 
+  ],
   "Input is too short, must be at least one character": [
-    "Input is too short, must be at least %(num)d characters", 
-    "", 
-    "", 
-    "", 
+    "Input is too short, must be at least %(num)d characters",
+    "",
+    "",
+    "",
     ""
-  ], 
+  ],
   "Link": [
-    null, 
+    null,
     "Ссылка"
-  ], 
+  ],
   "Loading...": [
-    null, 
+    null,
     "Идет загрузка..."
-  ], 
+  ],
   "Map": [
-    null, 
+    null,
     "ка́рта"
-  ], 
+  ],
   "No matches found": [
-    null, 
+    null,
     "Совпадений не найдено"
-  ], 
+  ],
   "Please Confirm Action": [
-    null, 
+    null,
     "Пожалуйста, подтвердите действие"
-  ], 
+  ],
+  "Please select the file to upload again": [
+    null,
+    "Пожалуйста, укажите файл для повторной загрузки "
+  ],
   "Remove": [
-    null, 
+    null,
     "Удалить"
-  ], 
+  ],
+  "Reorder resources": [
+    null,
+    "Упорядочить ресурсы"
+  ],
   "Resource uploaded": [
-    null, 
+    null,
     "Ресурс загружен"
-  ], 
+  ],
   "Save order": [
-    null, 
+    null,
     "Сохранить порядок"
-  ], 
+  ],
   "Saving...": [
-    null, 
+    null,
     "Сохранение..."
-  ], 
+  ],
   "Show more": [
-    null, 
+    null,
     "Показать еще"
-  ], 
+  ],
   "Start typing…": [
-    null, 
+    null,
     "Начните вводить ..."
-  ], 
+  ],
   "There are unsaved modifications to this form": [
-    null, 
+    null,
     "Есть несохраненные изменения в этой форме"
-  ], 
+  ],
   "There is no API data to load for this resource": [
-    null, 
+    null,
     "Нет API данных для загрузки для данного ресурса"
-  ], 
+  ],
   "URL": [
-    null, 
+    null,
     "URL"
-  ], 
+  ],
   "Unable to authenticate upload": [
-    null, 
+    null,
     "Не удается загрузить аутентификации"
-  ], 
+  ],
   "Unable to get data for uploaded file": [
-    null, 
+    null,
     "Невозможно получить данные для загружаемого файла"
-  ], 
+  ],
   "Unable to upload file": [
-    null, 
+    null,
     "Невозможно загрузить файл"
-  ], 
+  ],
   "Unfollow": [
-    null, 
+    null,
     "Не следовать"
-  ], 
+  ],
   "Upload": [
-    null, 
+    null,
     "Загрузка"
-  ], 
+  ],
   "Upload a file": [
-    null, 
+    null,
     "Вложить файл"
-  ], 
+  ],
   "Upload a file on your computer": [
-    null, 
+    null,
     "Загрузите файл с компьютера"
-  ], 
+  ],
   "You are uploading a file. Are you sure you want to navigate away and stop this upload?": [
-    null, 
+    null,
     "Вы загружаете файл. Вы уверены что хотите прервать ?"
-  ], 
-  "bar": [
-    "bars", 
-    "", 
-    "", 
-    "", 
-    ""
-  ], 
+  ],
   "error loading view": [
-    null, 
+    null,
     "Ошибка при загрузке просмотра"
-  ], 
-  "no %(attr)s translation": [
-    "no %(attr)s translations", 
-    "", 
-    "", 
-    "", 
-    ""
-  ], 
-  "no translation": [
-    "no translations", 
-    "", 
-    "", 
-    "", 
-    ""
   ]
 }
\ No newline at end of file
