@@ -9,13 +9,11 @@ $(function (){
           type : 'POST',
           data : {url:url, type:'page'},
           timeout : 300 });
-  $('a.resource-url-analytics').click(function (e){
+  $('a.resource-url-analytics').on('click', function (e){
     var url = $(e.target).closest('a').attr('href');
     $.ajax({url : $('body').data('site-root') + '_tracking',
             data : {url:url, type:'resource'},
             type : 'POST',
-            complete : function () {location.href = url;},
             timeout : 30});
-    e.preventDefault();
   });
 });
