@@ -1,10 +1,10 @@
 # encoding: utf-8
 
-'''**All migration scripts should have tests.**
+"""**All migration scripts should have tests.**
 
 .. todo::
 
    Write some tests for a migration script, and then use them as an example to
    fill out this guidelines section.
 
-'''
+"""
