@@ -1,10 +1,13 @@
 # encoding: utf-8
 
 import os
-import cgi
 import logging
+import html
+import io
+import datetime
 
 from flask import Blueprint, make_response
+import six
 from six import text_type
 from werkzeug.exceptions import BadRequest
 
@@ -12,6 +15,7 @@ import ckan.model as model
 from ckan.common import json, _, g, request
 from ckan.lib.helpers import url_for
 from ckan.lib.base import render
+from ckan.lib.i18n import get_locales_from_config
 
 from ckan.lib.navl.dictization_functions import DataError
 from ckan.logic import get_action, ValidationError, NotFound, NotAuthorized
@@ -36,6 +40,12 @@ API_MAX_VERSION = 3
 api = Blueprint(u'api', __name__, url_prefix=u'/api')
 
 
+def _json_serial(obj):
+    if isinstance(obj, (datetime.datetime, datetime.date)):
+        return obj.isoformat()
+    raise TypeError(u"Unhandled Object")
+
+
 def _finish(status_int, response_data=None,
             content_type=u'text', headers=None):
     u'''When a controller method has completed, call this method
@@ -63,6 +73,7 @@ def _finish(status_int, response_data=None,
         if content_type == u'json':
             response_msg = json.dumps(
                 response_data,
+                default=_json_serial,   # handle datetime objects
                 for_json=True)  # handle objects with for_json methods
         else:
             response_msg = response_data
@@ -70,7 +81,7 @@ def _finish(status_int, response_data=None,
         if (status_int == 200 and u'callback' in request.args and
                 request.method == u'GET'):
             # escape callback to remove '<', '&', '>' chars
-            callback = cgi.escape(request.args[u'callback'])
+            callback = html.escape(request.args[u'callback'])
             response_msg = _wrap_jsonp(callback, response_msg)
             headers[u'Content-Type'] = CONTENT_TYPES[u'javascript']
     return make_response((response_msg, status_int, headers))
@@ -162,7 +173,7 @@ def _get_request_data(try_url_params=False):
            item or a string otherwise
         '''
         out = {}
-        for key, value in multi_dict.to_dict(flat=False).iteritems():
+        for key, value in six.iteritems(multi_dict.to_dict(flat=False)):
             out[key] = value[0] if len(value) == 1 else value
         return out
 
@@ -172,10 +183,12 @@ def _get_request_data(try_url_params=False):
 
     request_data = {}
     if request.method in [u'POST', u'PUT'] and request.form:
-        if (len(request.form.values()) == 1 and
-                request.form.values()[0] in [u'1', u'']):
+        values = list(request.form.values())
+        if (len(values) == 1 and
+                values[0] in [u'1', u'']):
             try:
-                request_data = json.loads(request.form.keys()[0])
+                keys = list(request.form.keys())
+                request_data = json.loads(keys[0])
             except ValueError as e:
                 raise ValueError(
                     u'Error decoding JSON data. '
@@ -202,7 +215,7 @@ def _get_request_data(try_url_params=False):
     if request.method == u'PUT' and not request_data:
         raise ValueError(u'Invalid request. Please use the POST method for '
                          'your request')
-    for field_name, file_ in request.files.iteritems():
+    for field_name, file_ in six.iteritems(request.files):
         request_data[field_name] = file_
     log.debug(u'Request data extracted: %r', request_data)
 
@@ -419,12 +432,13 @@ def format_autocomplete(ver=API_REST_DEFAULT_VERSION):
 def user_autocomplete(ver=API_REST_DEFAULT_VERSION):
     q = request.args.get(u'q', u'')
     limit = request.args.get(u'limit', 20)
+    ignore_self = request.args.get(u'ignore_self', False)
     user_list = []
     if q:
         context = {u'model': model, u'session': model.Session,
                    u'user': g.user, u'auth_user_obj': g.userobj}
 
-        data_dict = {u'q': q, u'limit': limit}
+        data_dict = {u'q': q, u'limit': limit, u'ignore_self': ignore_self}
 
         user_list = get_action(u'user_autocomplete')(context, data_dict)
     return _finish_ok(user_list)
@@ -470,12 +484,16 @@ def snippet(snippet_path, ver=API_REST_DEFAULT_VERSION):
 
 
 def i18n_js_translations(lang, ver=API_REST_DEFAULT_VERSION):
+
+    if lang not in get_locales_from_config():
+        return _finish_bad_request(u'Unknown locale: {}'.format(lang))
+
     ckan_path = os.path.join(os.path.dirname(__file__), u'..')
     source = os.path.abspath(os.path.join(ckan_path, u'public',
                              u'base', u'i18n', u'{0}.js'.format(lang)))
     if not os.path.exists(source):
         return u'{}'
-    translations = json.load(open(source, u'r'))
+    translations = json.load(io.open(source, u'r', encoding=u'utf-8'))
     return _finish_ok(translations)
 
 
