@@ -2,10 +2,10 @@
 
 import logging
 import json
-import urlparse
 import datetime
 import time
 
+from six.moves.urllib.parse import urljoin
 from dateutil.parser import parse as parse_date
 
 import requests
@@ -14,7 +14,7 @@ import ckan.lib.helpers as h
 import ckan.lib.navl.dictization_functions
 import ckan.logic as logic
 import ckan.plugins as p
-from ckan.common import config
+from ckan.common import config, asint
 import ckanext.datapusher.logic.schema as dpschema
 import ckanext.datapusher.interfaces as interfaces
 
@@ -22,6 +22,8 @@ log = logging.getLogger(__name__)
 _get_or_bust = logic.get_or_bust
 _validate = ckan.lib.navl.dictization_functions.validate
 
+TIMEOUT = asint(config.get('ckan.requests.timeout', 5))
+
 
 def datapusher_submit(context, data_dict):
     ''' Submit a job to the datapusher. The datapusher is a service that
@@ -61,18 +63,16 @@ def datapusher_submit(context, data_dict):
 
     datapusher_url = config.get('ckan.datapusher.url')
 
-    site_url = h.url_for('/', qualified=True)
-
     callback_url_base = config.get('ckan.datapusher.callback_url_base')
     if callback_url_base:
-        callback_url = urlparse.urljoin(
+        site_url = callback_url_base
+        callback_url = urljoin(
             callback_url_base.rstrip('/'), '/api/3/action/datapusher_hook')
     else:
+        site_url = h.url_for('/', qualified=True)
         callback_url = h.url_for(
             '/api/3/action/datapusher_hook', qualified=True)
 
-    user = p.toolkit.get_action('user_show')(context, {'id': context['user']})
-
     for plugin in p.PluginImplementations(interfaces.IDataPusher):
         upload = plugin.can_upload(res_id)
         if not upload:
@@ -119,16 +119,24 @@ def datapusher_submit(context, data_dict):
         pass
 
     context['ignore_auth'] = True
+    # Use local session for task_status_update, so it can commit its own
+    # results without messing up with the parent session that contains pending
+    # updats of dataset/resource/etc.
+    context['session'] = context['model'].meta.create_local_session()
     p.toolkit.get_action('task_status_update')(context, task)
 
+    site_user = p.toolkit.get_action(
+        'get_site_user')({'ignore_auth': True}, {})
+
     try:
         r = requests.post(
-            urlparse.urljoin(datapusher_url, 'job'),
+            urljoin(datapusher_url, 'job'),
             headers={
                 'Content-Type': 'application/json'
             },
+            timeout=TIMEOUT,
             data=json.dumps({
-                'api_key': user['apikey'],
+                'api_key': site_user['apikey'],
                 'job_type': 'push_to_datastore',
                 'result_url': callback_url,
                 'metadata': {
@@ -151,9 +159,11 @@ def datapusher_submit(context, data_dict):
         raise p.toolkit.ValidationError(error)
 
     except requests.exceptions.HTTPError as e:
-        m = 'An Error occurred while sending the job: {0}'.format(e.message)
+        m = 'An Error occurred while sending the job: {0}'.format(str(e))
         try:
             body = e.response.json()
+            if body.get('error'):
+                m += ' ' + body['error']
         except ValueError:
             body = e.response.text
         error = {'message': m,
@@ -289,10 +299,12 @@ def datapusher_status(context, data_dict):
     job_detail = None
 
     if job_id:
-        url = urlparse.urljoin(datapusher_url, 'job' + '/' + job_id)
+        url = urljoin(datapusher_url, 'job' + '/' + job_id)
         try:
-            r = requests.get(url, headers={'Content-Type': 'application/json',
-                                           'Authorization': job_key})
+            r = requests.get(url,
+                             timeout=TIMEOUT,
+                             headers={'Content-Type': 'application/json',
+                                      'Authorization': job_key})
             r.raise_for_status()
             job_detail = r.json()
             for log in job_detail['logs']:
