@@ -1,5 +1,6 @@
 # encoding: utf-8
 
+import six
 from six import string_types
 
 import ckan
@@ -125,7 +126,7 @@ def translate_resource_data_dict(data_dict):
 
     # Get a simple flat list of all the terms to be translated, from the
     # flattened data dict.
-    terms = sets.Set()
+    terms = set()
     for (key, value) in flattened.items():
         if value in (None, True, False):
             continue
@@ -225,7 +226,7 @@ class MultilingualDataset(SingletonPlugin):
 
         ## translate rest
         all_terms = []
-        for key, value in search_data.iteritems():
+        for key, value in sorted(six.iteritems(search_data)):
             if key in KEYS_TO_IGNORE or key.startswith('title'):
                 continue
             if not isinstance(value, list):
@@ -243,11 +244,11 @@ class MultilingualDataset(SingletonPlugin):
 
         text_field_items['text_' + default_lang].extend(all_terms)
 
-        for translation in sorted(field_translations):
+        for translation in sorted(field_translations, key=lambda tr: all_terms.index(tr['term'])):
             lang_field = 'text_' + translation['lang_code']
             text_field_items[lang_field].append(translation['term_translation'])
 
-        for key, value in text_field_items.iteritems():
+        for key, value in six.iteritems(text_field_items):
             search_data[key] = ' '.join(value)
 
         return search_data
